#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TProfile.h"
#include "TF1.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph2D.h"
#include "TRandom.h"


using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.15);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}


void MatrixTests(int Npoint = 1000)
{
  HallA_style();


  //*******************  Tg Th parameters **********************************

  double a0000, a0001, a0002, a0010, a0011, a0012, a0020, a0021, a0100, a0101, a0102, a0110, a0111, a0120, a0200, a0201, a0210, a1000, a1001, a1010, a1100, a1200, a2000, a2001, a2010, a2100, a2200, a3000, a3100, a4000;

  //ifstream myfile("ParameterResultsThMatlabFinal.dat");
  ifstream myfile("ParameterResultsThMatlab.dat");
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >> a0000 >> a0001 >> a0002 >> a0010 >> a0011 >> a0012 >> a0020 >> a0021 >>  a0100 >> a0101 >> a0102 >> a0110 >> a0111 >> a0120 >> a0200 >> a0201 >> a0210 >> a1000 >> a1001 >> a1010 >> a1100 >> a1200 >> a2000 >> a2001 >> a2010 >> a2100 >> a2200 >> a3000 >> a3100 >> a4000;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<", a4000 = "<<a4000<<endl;

  //*******************  Tg Ph parameters **********************************

  double   b0000, b0001, b0002, b0003, b0004, b0011, b0012, b0013, b0020, b0021, b0022, b0030, b0031, b0040, b0101, b0102, b0103, b0110, b0111, b0112, b0121, b0130, b0201, b0210, b0211, b0220, b1000, b1001, b1002, b1003, b1010, b1011, b1012, b1020, b1021, b1030, b1100, b1101, b1102, b1110, b1111, b1200, b1201, b1210, b2000, b2001, b2002, b2010, b2011, b2020, b2100, b2101, b2110, b2200, b3000, b3001, b3010, b3100, b4001, b5001, b3101, b4101, b2201, b3201, b1103, b2103, b3020, b4020;

  ifstream myfile2 ("ParameterResultsPhiMatlab.dat");
  if (myfile2.is_open())
  {
    while (! myfile2.eof() )
    {
      

       myfile2 >> b0000 >> b0001 >> b0002 >> b0003 >> b0004 >> b0011 >> b0012 >> b0013 >> b0020 >> b0021 >> b0022 >> b0030 >> b0031 >> b0040 >> b0101 >> b0102 >> b0103 >> b0110 >> b0111 >> b0112 >> b0121 >> b0130 >> b0201 >> b0210 >> b0211 >> b0220 >> b1000 >> b1001 >> b1002 >> b1003 >> b1010 >> b1011 >> b1012 >> b1020 >> b1021 >> b1030 >> b1100 >> b1101 >> b1102 >> b1110 >> b1111 >> b1200 >> b1201 >> b1210 >> b2000 >> b2001 >> b2002 >> b2010 >> b2011 >> b2020 >> b2100 >> b2101 >> b2110 >> b2200 >> b3000 >> b3001 >> b3010 >> b3100 >> b4001 >> b5001 >> b3101 >> b4101 >> b2201 >> b3201 >> b1103 >> b2103 >> b3020 >> b4020 ;

    }
    myfile2.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: b0000 = "<<b0000<<", b0001 = "<<b0001<<", b0002 = "<<b0002<<endl;


  //*******************  TgY parameters **********************************

  double c0000, c0001, c0003, c0010, c0011, c0012, c0020, c0021, c0030, c0102, c0110, c0111, c0120, c0200, c0201, c0210, c1000, c1001, c1010, c1011, c1020, c1200, c2000, c2200, c3000;

  ifstream myfile3("ParameterResultsTgYMatlab.dat");
  if (myfile3.is_open())
  {
    while (! myfile3.eof() )
    {
      myfile3 >>  c0000 >> c0001 >> c0003 >> c0010 >> c0011 >> c0012 >> c0020 >> c0021 >> c0030 >> c0102 >> c0110 >> c0111 >> c0120 >> c0200 >> c0201 >> c0210 >> c1000 >> c1001 >> c1010 >> c1011 >> c1020 >> c1200 >> c2000 >> c2200 >> c3000;
    }
    myfile3.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: c0000 = "<<c0000<<", c0001 = "<<c0001<<endl;


  //*******************  TgDelta parameters **********************************

  double d0000, d1000, d0100, d2000, d2100, d2200, d0300, d1300, d2300, d0200, d0002, d1100, d0010, d0001, d0101, d3000, d1020, d0111, d0021, d1200, d0120, d0210, d0201, d0400, d0030, d0020, d1001, d1002, d0102, d1010, d2010, d1111, d4000, d3100 ,d2001, d3001, d2002, d0003, d1003;
  ifstream myfile4("ParameterResultsDeltaMatlab.dat");
  if (myfile4.is_open())
  {
    while (! myfile4.eof() )
    {
      myfile4 >>  d0000 >> d1000 >> d0100 >> d2000 >> d2100 >> d2200 >> d0300 >> d1300 >> d2300 >> d0200 >> d0002 >> d1100 >> d0010 >> d0001 >> d0101 >> d3000 >> d1020 >> d0111 >> d0021 >> d1200 >> d0120 >> d0210 >> d0201 >> d0400 >> d0030 >> d0020 >> d1001 >> d1002 >> d0102 >> d1010 >> d2010 >> d1111 >> d4000 >> d3100 >> d2001 >> d3001 >> d2002 >> d0003 >> d1003;
    }
    myfile4.close();
  }
  else cout << "Unable to open file"; 
  cout<<"Parameters: d0000 = "<<d0000<<", d1000 = "<<d1000<<endl;



  //*************************************************************************


  TGraph2D *grTgTh_FpxFpy = new TGraph2D();
  grTgTh_FpxFpy->SetNpx(200);
  grTgTh_FpxFpy->SetNpy(200);
  TGraph2D *grTgPh_FpxFpy = new TGraph2D();
  TGraph2D *grTgY_FpxFpy = new TGraph2D();
  TRandom *generator = new TRandom();


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(-0.8, 0.8);
    double y = generator->Uniform(-0.14, 0.14);
    double th = 0.0;
    double ph = 0.0;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    grTgTh_FpxFpy->SetPoint(i, x,y,TgTh);
    grTgPh_FpxFpy->SetPoint(i, x,y,TgPh);
    grTgY_FpxFpy->SetPoint(i, x,y,TgY);
     
  }

  TCanvas* c0 = new TCanvas("c0","c0",660,660);
  c0->Divide(2,2,0.001,0.001);
  c0->cd(1);
  gStyle->SetPalette(1);
  grTgTh_FpxFpy->Draw("COLZ");
  grTgTh_FpxFpy->SetTitle("TgTh Plot");
  grTgTh_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  grTgTh_FpxFpy->GetYaxis()->SetTitle("FpY [m]");

  c0->cd(2);
  gStyle->SetPalette(1);
  grTgPh_FpxFpy->Draw("COLZ");
  grTgPh_FpxFpy->SetTitle("TgPh Plot");
  grTgPh_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  grTgPh_FpxFpy->GetYaxis()->SetTitle("FpY [m]");
  c0->cd(3);
  gStyle->SetPalette(1);
  grTgY_FpxFpy->Draw("COLZ");
  grTgY_FpxFpy->SetTitle("TgY Plot");
  grTgY_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  grTgY_FpxFpy->GetYaxis()->SetTitle("FpY [m]");
  

  //*************************************************************************


  TGraph2D *grTgTh_FpxFpth = new TGraph2D();
  TGraph2D *grTgPh_FpxFpth = new TGraph2D();
  TGraph2D *grTgY_FpxFpth = new TGraph2D();


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(-0.8, 0.8);
    double y = 0.0;
    double th = generator->Uniform(-0.3, 0.3);
    double ph = 0.0;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    grTgTh_FpxFpth->SetPoint(i, x,th,TgTh);
    grTgPh_FpxFpth->SetPoint(i, x,th,TgPh);
    grTgY_FpxFpth->SetPoint(i, x,th,TgY);
     
  }

  TCanvas* c1 = new TCanvas("c1","c1",660,660);
  c1->Divide(2,2,0.001,0.001);
  c1->cd(1);
  gStyle->SetPalette(1);
  grTgTh_FpxFpth->Draw("COLZ");
  grTgTh_FpxFpth->SetTitle("TgTh Plot");
  grTgTh_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  grTgTh_FpxFpth->GetYaxis()->SetTitle("FpTh [m]");

  c1->cd(2);
  gStyle->SetPalette(1);
  grTgPh_FpxFpth->Draw("COLZ");
  grTgPh_FpxFpth->SetTitle("TgPh Plot");
  grTgPh_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  grTgPh_FpxFpth->GetYaxis()->SetTitle("FpTh [m]");
  c1->cd(3);
  gStyle->SetPalette(1);
  grTgY_FpxFpth->Draw("COLZ");
  grTgY_FpxFpth->SetTitle("TgY Plot");
  grTgY_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  grTgY_FpxFpth->GetYaxis()->SetTitle("FpTh [m]");

 //*************************************************************************


  TGraph2D *grTgTh_FpxFpph = new TGraph2D();
  TGraph2D *grTgPh_FpxFpph = new TGraph2D();
  TGraph2D *grTgY_FpxFpph = new TGraph2D();


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(-0.8, 0.8);
    double y = 0.0;
    double th = 0.0;
    double ph = generator->Uniform(-0.12, 0.12);

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    grTgTh_FpxFpph->SetPoint(i, x, ph,TgTh);
    grTgPh_FpxFpph->SetPoint(i, x, ph,TgPh);
    grTgY_FpxFpph->SetPoint(i, x, ph,TgY);
     
  }

  TCanvas* c2 = new TCanvas("c2","c2",660,660);
  c2->Divide(2,2,0.001,0.001);
  c2->cd(1);
  gStyle->SetPalette(1);
  grTgTh_FpxFpph->Draw("COLZ");
  grTgTh_FpxFpph->SetTitle("TgTh Plot");
  grTgTh_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  grTgTh_FpxFpph->GetYaxis()->SetTitle("FpPh [m]");

  c2->cd(2);
  gStyle->SetPalette(1);
  grTgPh_FpxFpph->Draw("COLZ");
  grTgPh_FpxFpph->SetTitle("TgPh Plot");
  grTgPh_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  grTgPh_FpxFpph->GetYaxis()->SetTitle("FpPh [m]");
  c2->cd(3);
  gStyle->SetPalette(1);
  grTgY_FpxFpph->Draw("COLZ");
  grTgY_FpxFpph->SetTitle("TgY Plot");
  grTgY_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  grTgY_FpxFpph->GetYaxis()->SetTitle("FpPh [m]");


 //*************************************************************************


  TGraph2D *grTgTh_FpyFpph = new TGraph2D();
  TGraph2D *grTgPh_FpyFpph = new TGraph2D();
  TGraph2D *grTgY_FpyFpph = new TGraph2D();


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = 0.0;
    double y = generator->Uniform(-0.14, 0.14);
    double th = 0.0;
    double ph = generator->Uniform(-0.12, 0.12);

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    grTgTh_FpyFpph->SetPoint(i, y, ph,TgTh);
    grTgPh_FpyFpph->SetPoint(i, y, ph,TgPh);
    grTgY_FpyFpph->SetPoint(i, y, ph,TgY);
     
  }

  TCanvas* c3 = new TCanvas("c3","c3",660,660);
  c3->Divide(2,2,0.001,0.001);
  c3->cd(1);
  gStyle->SetPalette(1);
  grTgTh_FpyFpph->Draw("COLZ");
  grTgTh_FpyFpph->SetTitle("TgTh Plot");
  grTgTh_FpyFpph->GetXaxis()->SetTitle("FpY [m]");
  grTgTh_FpyFpph->GetYaxis()->SetTitle("FpPh [m]");

  c3->cd(2);
  gStyle->SetPalette(1);
  grTgPh_FpyFpph->Draw("COLZ");
  grTgPh_FpyFpph->SetTitle("TgPh Plot");
  grTgPh_FpyFpph->GetXaxis()->SetTitle("FpY [m]");
  grTgPh_FpyFpph->GetYaxis()->SetTitle("FpPh [m]");
  c3->cd(3);
  gStyle->SetPalette(1);
  grTgY_FpyFpph->Draw("COLZ");
  grTgY_FpyFpph->SetTitle("TgY Plot");
  grTgY_FpyFpph->GetXaxis()->SetTitle("FpY [m]");
  grTgY_FpyFpph->GetYaxis()->SetTitle("FpPh [m]");

 //*************************************************************************


  TGraph2D *grTgTh_FpthFpph = new TGraph2D();
  TGraph2D *grTgPh_FpthFpph = new TGraph2D();
  TGraph2D *grTgY_FpthFpph = new TGraph2D();


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = 0.0;
    double y = 0.0;
    double th = generator->Uniform(-0.3, 0.3);
    double ph = generator->Uniform(-0.12, 0.12);

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    grTgTh_FpthFpph->SetPoint(i, th, ph,TgTh);
    grTgPh_FpthFpph->SetPoint(i, th, ph,TgPh);
    grTgY_FpthFpph->SetPoint(i, th, ph,TgY);
     
  }

  TCanvas* c4 = new TCanvas("c4","c4",660,660);
  c4->Divide(2,2,0.001,0.001);
  c4->cd(1);
  gStyle->SetPalette(1);
  grTgTh_FpthFpph->Draw("COLZ");
  grTgTh_FpthFpph->SetTitle("TgTh Plot");
  grTgTh_FpthFpph->GetXaxis()->SetTitle("FpTh [1]");
  grTgTh_FpthFpph->GetYaxis()->SetTitle("FpPh [1]");

  c4->cd(2);
  gStyle->SetPalette(1);
  grTgPh_FpthFpph->Draw("COLZ");
  grTgPh_FpthFpph->SetTitle("TgPh Plot");
  grTgPh_FpthFpph->GetXaxis()->SetTitle("FpTh [1]");
  grTgPh_FpthFpph->GetYaxis()->SetTitle("FpPh [1]");
  c4->cd(3);
  gStyle->SetPalette(1);
  grTgY_FpthFpph->Draw("COLZ");
  grTgY_FpthFpph->SetTitle("TgY Plot");
  grTgY_FpthFpph->GetXaxis()->SetTitle("FpTh [m]");
  grTgY_FpthFpph->GetYaxis()->SetTitle("FpPh [m]");

  



}
