#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TProfile.h"
#include "TF1.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph2D.h"
#include "TRandom.h"


using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.15);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}


void MatrixTests2(int Npoint = 3000000, int useLimits=0)
{
  HallA_style();


  //*******************  Tg Th parameters **********************************

  double a0000, a0001, a0002, a0010, a0011, a0012, a0020, a0021, a0100, a0101, a0102, a0110, a0111, a0120, a0200, a0201, a0210, a1000, a1001, a1010, a1100, a1200, a2000, a2001, a2010, a2100, a2200, a3000, a3100, a4000;

  //ifstream myfile("ParameterResultsThMatlabFinal.dat");
  ifstream myfile("ParameterResultsThMatlab.dat");
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >> a0000 >> a0001 >> a0002 >> a0010 >> a0011 >> a0012 >> a0020 >> a0021 >>  a0100 >> a0101 >> a0102 >> a0110 >> a0111 >> a0120 >> a0200 >> a0201 >> a0210 >> a1000 >> a1001 >> a1010 >> a1100 >> a1200 >> a2000 >> a2001 >> a2010 >> a2100 >> a2200 >> a3000 >> a3100 >> a4000;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<", a4000 = "<<a4000<<endl;

  //*******************  Tg Ph parameters **********************************

  double   b0000, b0001, b0002, b0003, b0004, b0011, b0012, b0013, b0020, b0021, b0022, b0030, b0031, b0040, b0101, b0102, b0103, b0110, b0111, b0112, b0121, b0130, b0201, b0210, b0211, b0220, b1000, b1001, b1002, b1003, b1010, b1011, b1012, b1020, b1021, b1030, b1100, b1101, b1102, b1110, b1111, b1200, b1201, b1210, b2000, b2001, b2002, b2010, b2011, b2020, b2100, b2101, b2110, b2200, b3000, b3001, b3010, b3100, b4001, b5001, b3101, b4101, b2201, b3201, b1103, b2103, b3020, b4020;

  ifstream myfile2 ("ParameterResultsPhiMatlab.dat");
  if (myfile2.is_open())
  {
    while (! myfile2.eof() )
    {
      

       myfile2 >> b0000 >> b0001 >> b0002 >> b0003 >> b0004 >> b0011 >> b0012 >> b0013 >> b0020 >> b0021 >> b0022 >> b0030 >> b0031 >> b0040 >> b0101 >> b0102 >> b0103 >> b0110 >> b0111 >> b0112 >> b0121 >> b0130 >> b0201 >> b0210 >> b0211 >> b0220 >> b1000 >> b1001 >> b1002 >> b1003 >> b1010 >> b1011 >> b1012 >> b1020 >> b1021 >> b1030 >> b1100 >> b1101 >> b1102 >> b1110 >> b1111 >> b1200 >> b1201 >> b1210 >> b2000 >> b2001 >> b2002 >> b2010 >> b2011 >> b2020 >> b2100 >> b2101 >> b2110 >> b2200 >> b3000 >> b3001 >> b3010 >> b3100 >> b4001 >> b5001 >> b3101 >> b4101 >> b2201 >> b3201 >> b1103 >> b2103 >> b3020 >> b4020 ;

    }
    myfile2.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: b0000 = "<<b0000<<", b0001 = "<<b0001<<", b0002 = "<<b0002<<endl;


  //*******************  TgY parameters **********************************

  double c0000, c0001, c0003, c0010, c0011, c0012, c0020, c0021, c0030, c0102, c0110, c0111, c0120, c0200, c0201, c0210, c1000, c1001, c1010, c1011, c1020, c1200, c2000, c2200, c3000;

  ifstream myfile3("ParameterResultsTgYMatlabSVD.dat");
  if (myfile3.is_open())
  {
    while (! myfile3.eof() )
    {
      myfile3 >>  c0000 >> c0001 >> c0003 >> c0010 >> c0011 >> c0012 >> c0020 >> c0021 >> c0030 >> c0102 >> c0110 >> c0111 >> c0120 >> c0200 >> c0201 >> c0210 >> c1000 >> c1001 >> c1010 >> c1011 >> c1020 >> c1200 >> c2000 >> c2200 >> c3000;
    }
    myfile3.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: c0000 = "<<c0000<<", c0001 = "<<c0001<<endl;


  //*******************  TgDelta parameters **********************************

  double d0000, d1000, d0100, d2000, d2100, d2200, d0300, d1300, d2300, d0200, d0002, d1100, d0010, d0001, d0101, d3000, d1020, d0111, d0021, d1200, d0120, d0210, d0201, d0400, d0030, d0020, d1001, d1002, d0102, d1010, d2010, d1111, d4000, d3100 ,d2001, d3001, d2002, d0003, d1003;
  ifstream myfile4("ParameterResultsDeltaMatlab.dat");
  if (myfile4.is_open())
  {
    while (! myfile4.eof() )
    {
      myfile4 >>  d0000 >> d1000 >> d0100 >> d2000 >> d2100 >> d2200 >> d0300 >> d1300 >> d2300 >> d0200 >> d0002 >> d1100 >> d0010 >> d0001 >> d0101 >> d3000 >> d1020 >> d0111 >> d0021 >> d1200 >> d0120 >> d0210 >> d0201 >> d0400 >> d0030 >> d0020 >> d1001 >> d1002 >> d0102 >> d1010 >> d2010 >> d1111 >> d4000 >> d3100 >> d2001 >> d3001 >> d2002 >> d0003 >> d1003;
    }
    myfile4.close();
  }
  else cout << "Unable to open file"; 
  cout<<"Parameters: d0000 = "<<d0000<<", d1000 = "<<d1000<<endl;



  //*************************************************************************


  TRandom *generator = new TRandom();
  double xLimits[2] = {-0.8, 0.8};
  double yLimits[2] = {-0.14, 0.14};
  double thLimits[2] = {-0.3, 0.3};
  double phLimits[2] = {-0.12, 0.14};

  double TgYLimits = 0.3;
  double TgThLimits = 0.3;
  double TgPhLimits = 0.2;


 //******************************* XY ******************************************

  TH2D *hTgTh_FpxFpy = new TH2D("hTgTh_FpxFpy", "hTgTh_FpxFpy", 500, xLimits[0], xLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgPh_FpxFpy = new TH2D("hTgPh_FpxFpy", "hTgPh_FpxFpy", 500, xLimits[0], xLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgY_FpxFpy = new TH2D("hTgY_FpxFpy", "hTgY_FpxFpy", 500, xLimits[0], xLimits[1], 500, yLimits[0], yLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(xLimits[0], xLimits[1]);
    double y = generator->Uniform(yLimits[0], yLimits[1]);
    double th = 0.0;
    double ph = 0.0;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpxFpy->SetBinContent(hTgTh_FpxFpy->FindBin(x,y),TgTh);
    hTgPh_FpxFpy->SetBinContent(hTgPh_FpxFpy->FindBin(x,y),TgPh);
    hTgY_FpxFpy->SetBinContent(hTgY_FpxFpy->FindBin(x,y),TgY);
     
  }

  TCanvas* c1 = new TCanvas("c1","c1",860,660);
  c1->Divide(2,2,0.001,0.001);
  c1->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpxFpy->SetMaximum(TgThLimits);
    hTgTh_FpxFpy->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpxFpy->Draw("COLZ");
  hTgTh_FpxFpy->SetTitle("TgTh(x,y)");
  hTgTh_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  hTgTh_FpxFpy->GetYaxis()->SetTitle("FpY [m]");

  c1->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpxFpy->SetMaximum(TgPhLimits);
    hTgPh_FpxFpy->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpxFpy->Draw("COLZ");
  hTgPh_FpxFpy->SetTitle("TgPh(x,y)");
  hTgPh_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  hTgPh_FpxFpy->GetYaxis()->SetTitle("FpY [m]");
  c1->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpxFpy->SetMaximum(TgYLimits);
    hTgY_FpxFpy->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpxFpy->Draw("COLZ");
  hTgY_FpxFpy->SetTitle("TgY(x,y)");
  hTgY_FpxFpy->GetXaxis()->SetTitle("FpX [m]");
  hTgY_FpxFpy->GetYaxis()->SetTitle("FpY [m]");

  TString FigureName;
  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",1,useLimits);
  c1->Print(FigureName.Data());

  
 //******************************* XTH ******************************************

  TH2D *hTgTh_FpxFpth = new TH2D("hTgTh_FpxFpth", "hTgTh_FpxFpth", 500, xLimits[0], xLimits[1], 500, thLimits[0], thLimits[1]);
  TH2D *hTgPh_FpxFpth = new TH2D("hTgPh_FpxFpth", "hTgPh_FpxFpth", 500, xLimits[0], xLimits[1], 500, thLimits[0], thLimits[1]);
  TH2D *hTgY_FpxFpth = new TH2D("hTgY_FpxFpth", "hTgY_FpxFpth", 500, xLimits[0], xLimits[1], 500, thLimits[0], thLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(xLimits[0], xLimits[1]);
    double y = 0.0;
    double th = generator->Uniform(thLimits[0], thLimits[1]);;
    double ph = 0.0;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpxFpth->SetBinContent(hTgTh_FpxFpth->FindBin(x,th),TgTh);
    hTgPh_FpxFpth->SetBinContent(hTgPh_FpxFpth->FindBin(x,th),TgPh);
    hTgY_FpxFpth->SetBinContent(hTgY_FpxFpth->FindBin(x,th),TgY);
     
  }

  TCanvas* c2 = new TCanvas("c2","c2",860,660);
  c2->Divide(2,2,0.001,0.001);
  c2->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpxFpth->SetMaximum(TgThLimits);
    hTgTh_FpxFpth->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpxFpth->Draw("COLZ");
  hTgTh_FpxFpth->SetTitle("TgTh(x,th)");
  hTgTh_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  hTgTh_FpxFpth->GetYaxis()->SetTitle("FpTh [1]");

  c2->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpxFpth->SetMaximum(TgPhLimits);
    hTgPh_FpxFpth->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpxFpth->Draw("COLZ");
  hTgPh_FpxFpth->SetTitle("TgPh(x,th)");
  hTgPh_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  hTgPh_FpxFpth->GetYaxis()->SetTitle("FpTh [1]");
  c2->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpxFpth->SetMaximum(TgYLimits);
    hTgY_FpxFpth->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpxFpth->Draw("COLZ");
  hTgY_FpxFpth->SetTitle("TgY(x,th)");
  hTgY_FpxFpth->GetXaxis()->SetTitle("FpX [m]");
  hTgY_FpxFpth->GetYaxis()->SetTitle("FpTh [1]");

  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",2,useLimits);
  c2->Print(FigureName.Data());

 //******************************* XPH ******************************************

  TH2D *hTgTh_FpxFpph = new TH2D("hTgTh_FpxFpph", "hTgTh_FpxFpph", 500, xLimits[0], xLimits[1], 500, phLimits[0], phLimits[1]);
  TH2D *hTgPh_FpxFpph = new TH2D("hTgPh_FpxFpph", "hTgPh_FpxFpph", 500, xLimits[0], xLimits[1], 500, phLimits[0], phLimits[1]);
  TH2D *hTgY_FpxFpph = new TH2D("hTgY_FpxFpph", "hTgY_FpxFpph", 500, xLimits[0], xLimits[1], 500, phLimits[0], phLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = generator->Uniform(xLimits[0], xLimits[1]);
    double y = 0.0;
    double th = 0.0;
    double ph = generator->Uniform(phLimits[0], phLimits[1]);

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpxFpph->SetBinContent(hTgTh_FpxFpph->FindBin(x,ph),TgTh);
    hTgPh_FpxFpph->SetBinContent(hTgPh_FpxFpph->FindBin(x,ph),TgPh);
    hTgY_FpxFpph->SetBinContent(hTgY_FpxFpph->FindBin(x,ph),TgY);
     
  }

  TCanvas* c3 = new TCanvas("c3","c3",860,660);
  c3->Divide(2,2,0.001,0.001);
  c3->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpxFpph->SetMaximum(TgThLimits);
    hTgTh_FpxFpph->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpxFpph->Draw("COLZ");
  hTgTh_FpxFpph->SetTitle("TgTh(x,ph)");
  hTgTh_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  hTgTh_FpxFpph->GetYaxis()->SetTitle("FpPh [1]");

  c3->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpxFpph->SetMaximum(TgPhLimits);
    hTgPh_FpxFpph->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpxFpph->Draw("COLZ");
  hTgPh_FpxFpph->SetTitle("TgPh(x,ph)");
  hTgPh_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  hTgPh_FpxFpph->GetYaxis()->SetTitle("FpPh [1]");

  c3->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpxFpph->SetMaximum(TgYLimits);
    hTgY_FpxFpph->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpxFpph->Draw("COLZ");
  hTgY_FpxFpph->SetTitle("TgY(x,y)");
  hTgY_FpxFpph->GetXaxis()->SetTitle("FpX [m]");
  hTgY_FpxFpph->GetYaxis()->SetTitle("FpPh [1]");

  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",3,useLimits);
  c3->Print(FigureName.Data());

 //******************************* PHY ******************************************

  TH2D *hTgTh_FpphFpy = new TH2D("hTgTh_FpphFpy", "hTgTh_FpphFpy", 500, phLimits[0], phLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgPh_FpphFpy = new TH2D("hTgPh_FpphFpy", "hTgPh_FpphFpy", 500, phLimits[0], phLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgY_FpphFpy = new TH2D("hTgY_FpphFpy", "hTgY_FpphFpy", 500, phLimits[0], phLimits[1], 500, yLimits[0], yLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = 0.0;
    double y = generator->Uniform(yLimits[0], yLimits[1]);
    double th = 0.0;
    double ph = generator->Uniform(phLimits[0], phLimits[1]);;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpphFpy->SetBinContent(hTgTh_FpphFpy->FindBin(ph,y),TgTh);
    hTgPh_FpphFpy->SetBinContent(hTgPh_FpphFpy->FindBin(ph,y),TgPh);
    hTgY_FpphFpy->SetBinContent(hTgY_FpphFpy->FindBin(ph,y),TgY);
     
  }

  TCanvas* c4 = new TCanvas("c4","c4",860,660);
  c4->Divide(2,2,0.001,0.001);
  c4->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpphFpy->SetMaximum(TgThLimits);
    hTgTh_FpphFpy->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpphFpy->Draw("COLZ");
  hTgTh_FpphFpy->SetTitle("TgTh(ph,y)");
  hTgTh_FpphFpy->GetXaxis()->SetTitle("FpPh [1]");
  hTgTh_FpphFpy->GetYaxis()->SetTitle("FpY [m]");

  c4->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpphFpy->SetMaximum(TgPhLimits);
    hTgPh_FpphFpy->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpphFpy->Draw("COLZ");
  hTgPh_FpphFpy->SetTitle("TgPh(ph,y)");
  hTgPh_FpphFpy->GetXaxis()->SetTitle("FpPh [1]");
  hTgPh_FpphFpy->GetYaxis()->SetTitle("FpY [m]");
  c4->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpphFpy->SetMaximum(TgYLimits);
    hTgY_FpphFpy->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpphFpy->Draw("COLZ");
  hTgY_FpphFpy->SetTitle("TgY(ph,y)");
  hTgY_FpphFpy->GetXaxis()->SetTitle("FpPh [1]");
  hTgY_FpphFpy->GetYaxis()->SetTitle("FpY [m]");

  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",4,useLimits);
  c4->Print(FigureName.Data());


 //******************************* PHTH ******************************************

  TH2D *hTgTh_FpphFpth = new TH2D("hTgTh_FpphFpth", "hTgTh_FpphFpth", 500, phLimits[0], phLimits[1], 500, thLimits[0], thLimits[1]);
  TH2D *hTgPh_FpphFpth = new TH2D("hTgPh_FpphFpth", "hTgPh_FpphFpth", 500, phLimits[0], phLimits[1], 500, thLimits[0], thLimits[1]);
  TH2D *hTgY_FpphFpth = new TH2D("hTgY_FpphFpth", "hTgY_FpphFpth", 500, phLimits[0], phLimits[1], 500, thLimits[0], thLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = 0.0;
    double y = 0.0;
    double th = generator->Uniform(thLimits[0], thLimits[1]);
    double ph = generator->Uniform(phLimits[0], phLimits[1]);;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpphFpth->SetBinContent(hTgTh_FpphFpth->FindBin(ph,th),TgTh);
    hTgPh_FpphFpth->SetBinContent(hTgPh_FpphFpth->FindBin(ph,th),TgPh);
    hTgY_FpphFpth->SetBinContent(hTgY_FpphFpth->FindBin(ph,th),TgY);
     
  }

  TCanvas* c5 = new TCanvas("c5","c5",860,660);
  c5->Divide(2,2,0.001,0.001);
  c5->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpphFpth->SetMaximum(TgThLimits);
    hTgTh_FpphFpth->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpphFpth->Draw("COLZ");
  hTgTh_FpphFpth->SetTitle("TgTh(ph,th)");
  hTgTh_FpphFpth->GetXaxis()->SetTitle("FpPh [1]");
  hTgTh_FpphFpth->GetYaxis()->SetTitle("FpTh [1]");

  c5->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpphFpth->SetMaximum(TgPhLimits);
    hTgPh_FpphFpth->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpphFpth->Draw("COLZ");
  hTgPh_FpphFpth->SetTitle("TgPh(ph,th)");
  hTgPh_FpphFpth->GetXaxis()->SetTitle("FpPh [1]");
  hTgPh_FpphFpth->GetYaxis()->SetTitle("FpTh [1]");

  c5->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpphFpth->SetMaximum(TgYLimits);
    hTgY_FpphFpth->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpphFpth->Draw("COLZ");
  hTgY_FpphFpth->SetTitle("TgY(ph,th)");
  hTgY_FpphFpth->GetXaxis()->SetTitle("FpPh [1]");
  hTgY_FpphFpth->GetYaxis()->SetTitle("FpTh [1]");

  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",5,useLimits);
  c5->Print(FigureName.Data());


 //******************************* THY ******************************************

  TH2D *hTgTh_FpthFpy = new TH2D("hTgTh_FpthFpy", "hTgTh_FpthFpy", 500, thLimits[0], thLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgPh_FpthFpy = new TH2D("hTgPh_FpthFpy", "hTgPh_FpthFpy", 500, thLimits[0], thLimits[1], 500, yLimits[0], yLimits[1]);
  TH2D *hTgY_FpthFpy = new TH2D("hTgY_FpthFpy", "hTgY_FpthFpy", 500, thLimits[0], thLimits[1], 500, yLimits[0], yLimits[1]);


  for (int i = 0; i<Npoint; i++){

    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = 0.0;
    double y = generator->Uniform(yLimits[0], yLimits[1]);
    double th = generator->Uniform(thLimits[0], thLimits[1]);
    double ph = 0.0;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);


    hTgTh_FpthFpy->SetBinContent(hTgTh_FpthFpy->FindBin(th,y),TgTh);
    hTgPh_FpthFpy->SetBinContent(hTgPh_FpthFpy->FindBin(th,y),TgPh);
    hTgY_FpthFpy->SetBinContent(hTgY_FpthFpy->FindBin(th,y),TgY);
     
  }

  TCanvas* c6 = new TCanvas("c6","c6",860,660);
  c6->Divide(2,2,0.001,0.001);
  c6->cd(1);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgTh_FpthFpy->SetMaximum(TgThLimits);
    hTgTh_FpthFpy->SetMinimum(-1.0*TgThLimits);
  }
  hTgTh_FpthFpy->Draw("COLZ");
  hTgTh_FpthFpy->SetTitle("TgTh(th,y)");
  hTgTh_FpthFpy->GetXaxis()->SetTitle("FpTh [1]");
  hTgTh_FpthFpy->GetYaxis()->SetTitle("FpY [m]");

  c6->cd(2);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgPh_FpthFpy->SetMaximum(TgPhLimits);
    hTgPh_FpthFpy->SetMinimum(-1.0*TgPhLimits);
  }
  hTgPh_FpthFpy->Draw("COLZ");
  hTgPh_FpthFpy->SetTitle("TgPh(th,y)");
  hTgPh_FpthFpy->GetXaxis()->SetTitle("FpTh [1]");
  hTgPh_FpthFpy->GetYaxis()->SetTitle("FpY [m]");
  c6->cd(3);
  gStyle->SetPalette(1);
  if (useLimits>0){
    hTgY_FpthFpy->SetMaximum(TgYLimits);
    hTgY_FpthFpy->SetMinimum(-1.0*TgYLimits);
  }
  hTgY_FpthFpy->Draw("COLZ");
  hTgY_FpthFpy->SetTitle("TgY(th,y)");
  hTgY_FpthFpy->GetXaxis()->SetTitle("FpTh [1]");
  hTgY_FpthFpy->GetYaxis()->SetTitle("FpY [m]");

  FigureName.Form("figure_MatrixTests_%d_Limits%d.png",6,useLimits);
  c6->Print(FigureName.Data());






}
