#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"


using namespace std;

int ApplyCuts( double x, double y, double th, double ph, double Tk1, double Tn1, double Tk2, double Tn2, double Pk1, double Pn1, double Pk2, double Pn2 ){

   if ( y > Pn1 + ph*Pk1 && y < Pn2 + ph*Pk2 && x > Tn1 + th*Tk1 && x < Tn2 + th*Tk2 ) return 1;
   else return 0;
}


void TargetPhCalibration(int runnumber=0, double MAXnum=-1)
{

  double in2m = 0.0254;
  double OffsetX = 0.0; // this is stated in LABCS
  double OffsetY = 0.0; // this is stated in LABCS
  double OffsetZ = 0.0288454; // this is stated in LABCS
  

  Double_t BB_gold_y;
  Double_t BB_gold_th;
  Double_t BB_gold_ph;
  Double_t BB_gold_dp;
  Double_t BB_tr_x[10];
  Double_t BB_tr_y[10];
  Double_t BB_tr_th[10];
  Double_t BB_tr_ph[10];
  Double_t scattering_angle;
  Double_t rb_x;
  Double_t rb_y;
  Double_t ReactPt_L_z;
  Double_t DL_t3[100];


  TChain *t1 =new TChain("tout");

  TString name;
  name.Form("./ROOTResults/eed_Test_GoodEvents_%d.root", runnumber);
  t1->Add(name.Data());
 
  assert(t1);

  t1->SetBranchAddress( "BB.gold.y", &BB_gold_y);
  t1->SetBranchAddress( "BB.gold.th", &BB_gold_th);
  t1->SetBranchAddress( "BB.gold.ph", &BB_gold_ph);
  t1->SetBranchAddress( "BB.gold.dp", &BB_gold_dp);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "scattering_angle", &scattering_angle);
  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);
  t1->SetBranchAddress( "rb.x", &rb_x);
  t1->SetBranchAddress( "rb.y", &rb_y);
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  const int PN = 71;

  TString PointNames[PN] = { "m5B",
			     "m4A",	
			     "m4B",
			     "m4C",
			     "m4D",
			     "m4E",
			     "m4F",
			     "m3A",
			     "m3D",
			     "m2A",
			     "m2B",
			     "m2C",
			     "m2D",
			     "m2E",
			     "m2F",
			     "m1A",
			     "m1B",
			     "m1C",
			     "m1D",
			     "m1E",
			     "m1F",
			     "p0A",
			     "p0B",
			     "p0C",
			     "p0D",
			     "p0E",
			     "p0F",
			     "p1A",
			     "p1B",
			     "p1C",
			     "p1D",
			     "p1E",
			     "p1F",
			     "p2A",
			     "p2B",
			     "p2C",
			     "p2D",
			     "p2E",
			     "p2F",
			     "p3B",
			     "p3F",
			     "p4B",
			     "p4C",
			     "p4D",
			     "p4E",
			     "p4F",
			     "p5C",
			     "p5D",
			     "p5EF",
			     "m5A",
			     "m5C",
			     "m5D",
			     "m5E",
			     "m5F",
			     "p3A",
			     "p4A",
			     "p6A",
			     "p6B",
			     "p6C",
			     "p6D",
			     "p6E",
			     "p6F",
			     "p6G",
			     "p5A",
			     "p5B",
			     "m6B",
			     "m6C",
			     "m6D",
			     "m6E",
			     "m6F",
			     "m6A"
				 };
  double ThFac = 0.04925;
  double ThWidthFac = 0.00952;
  double PhFac = 0.03810;

  double FPCuts[PN][10] = {
    { 0.360, 0.95,  0.410, 0.95, -0.13, 1.82, -0.095, 1.82, -5*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //-5B  //0
    { 0.275, 0.95,  0.330, 0.95, -0.16, 1.76, -0.125, 1.76, -4*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //-4A  //1  
    { 0.275, 0.95,  0.330, 0.95, -0.12, 1.73, -0.085, 1.73, -4*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //-4B  //2
    { 0.275, 0.95,  0.330, 0.95, -0.085, 1.73, -0.05, 1.73, -4*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //-4C  //3
    { 0.275, 0.95,  0.330, 0.95, -0.05, 1.73, -0.01, 1.73, -4*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //-4D  //4
    { 0.275, 0.95,  0.330, 0.95, -0.005, 1.73, 0.03, 1.73, -4*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //-4E  //5
    { 0.275, 0.95,  0.330, 0.95,  0.035, 1.73, 0.075, 1.73, -4*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //-4F  //6
    { 0.190, 0.95,  0.250, 0.95, -0.16, 1.7, -0.125, 1.7, -3*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //-3A  //7
    { 0.190, 0.95,  0.250, 0.95, -0.045, 1.68, -0.01, 1.68, -3*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //-3D  //8
    { 0.110, 0.92,  0.160, 0.92, -0.16, 1.64, -0.12, 1.64, -2*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //-2A  //9
    { 0.110, 0.92,  0.160, 0.92, -0.115, 1.64, -0.085, 1.64, -2*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //-2B  //10
    { 0.110, 0.92,  0.160, 0.92, -0.08, 1.64, -0.045, 1.64, -2*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //-2C  //11
    { 0.110, 0.92,  0.160, 0.92, -0.045, 1.64, -0.01, 1.64, -2*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //-2D  //12
    { 0.110, 0.92,  0.160, 0.92, -0.005, 1.64, 0.03, 1.64, -2*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //-2E  //13
    { 0.110, 0.92,  0.160, 0.92,  0.03, 1.64, 0.075, 1.64, -2*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //-2F  //14
    { 0.025, 0.85,  0.075, 0.85, -0.155, 1.62, -0.125, 1.62, -1*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //-1A  //15
    { 0.025, 0.85,  0.075, 0.85, -0.115, 1.62, -0.085, 1.62, -1*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //-1B  //16
    { 0.025, 0.85,  0.075, 0.85, -0.08, 1.62, -0.045, 1.62, -1*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //-1C  //17
    { 0.025, 0.85,  0.075, 0.85, -0.04, 1.62, -0.005, 1.62, -1*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //-1D  //18
    { 0.025, 0.85,  0.075, 0.85, -0.005, 1.62, 0.03, 1.62, -1*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //-1E  //19
    { 0.025, 0.85,  0.075, 0.85,  0.035, 1.62, 0.075, 1.62, -1*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //-1F  //20
    { -0.050, 0.85,  0.000, 0.80, -0.15, 1.6, -0.125, 1.6, 0*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //0A  //21
    { -0.050, 0.85,  0.000, 0.80, -0.12, 1.6, -0.085, 1.6, 0*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //0B  //22
    { -0.050, 0.85,  0.000, 0.80, -0.08, 1.6, -0.045, 1.6, 0*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //0C  //23
    { -0.050, 0.85,  0.000, 0.80, -0.04, 1.6, -0.005, 1.6, 0*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //0D  //24
    { -0.050, 0.85,  0.000, 0.80,  0.0, 1.6, 0.03, 1.6, 0*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //0E  //25
    { -0.050, 0.85,  0.000, 0.80,  0.035, 1.6, 0.075, 1.6, 0*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //0F  //26
    { -0.125, 0.85, -0.080, 0.85, -0.16, 1.65, -0.125, 1.65, 1*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //1A  //27
    { -0.125, 0.85, -0.080, 0.85, -0.12, 1.65, -0.085, 1.65, 1*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //1B  //28
    { -0.125, 0.85, -0.080, 0.85, -0.08, 1.6, -0.045, 1.6, 1*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //1C  //29
    { -0.125, 0.85, -0.080, 0.85, -0.04, 1.6, -0.005, 1.6, 1*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //1D  //30
    { -0.125, 0.85, -0.080, 0.85,  0.0, 1.6, 0.03, 1.6, 1*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //1E  //31
    { -0.125, 0.85, -0.080, 0.85,  0.035, 1.6, 0.075, 1.6, 1*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //1F  //32
    { -0.210, 0.82, -0.160, 0.82, -0.155, 1.6, -0.125, 1.6, 2*ThFac + OffsetY, -3.0*PhFac + OffsetX  },  //2A  //33
    { -0.210, 0.82, -0.160, 0.82, -0.12, 1.6, -0.085, 1.6, 2*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //2B  //34
    { -0.210, 0.82, -0.160, 0.82, -0.08, 1.58, -0.045, 1.58, 2*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //2C  //35
    { -0.210, 0.82, -0.160, 0.82, -0.04, 1.6, -0.005, 1.6, 2*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //2D  //36
    { -0.210, 0.82, -0.160, 0.82,  0.00, 1.6, 0.03, 1.6, 2*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //2E  //37
    { -0.210, 0.82, -0.160, 0.82,  0.035, 1.6, 0.075, 1.6, 2*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //2F  //38
    { -0.280, 0.80, -0.240, 0.80, -0.12, 1.6, -0.08, 1.6, 3*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //3B  //39
    { -0.280, 0.80, -0.240, 0.80,  0.038, 1.6, 0.078, 1.6, 3*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //3F  //40
    { -0.360, 0.78, -0.310, 0.70, -0.105, 1.4, -0.075, 1.4, 4*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //4B  //41
    { -0.360, 0.78, -0.310, 0.70, -0.07, 1.4, -0.04, 1.4, 4*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //4C  //42
    { -0.360, 0.78, -0.310, 0.70, -0.035, 1.4, -0.005, 1.4, 4*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //4D  //43
    { -0.360, 0.78, -0.310, 0.70,  0.0, 1.4, 0.03, 1.4, 4*ThFac + OffsetY, 1.0*PhFac + OffsetX },  //4E  //44
    { -0.360, 0.78, -0.310, 0.70,  0.035, 1.4, 0.075, 1.4, 4*ThFac + OffsetY, 2.0*PhFac + OffsetX },  //4F  //45
    { -0.430, 0.82, -0.390, 0.82, -0.06, 1.25, -0.04, 1.25, 5*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //5C  //46
    { -0.430, 0.82, -0.390, 0.82, -0.03, 1.25, -0.01, 1.25, 5*ThFac + OffsetY, 0.0*PhFac + OffsetX },  //5D  //47
    { -0.430, 0.82, -0.390, 0.82, 0.0, 1.25, 0.03, 1.25, 5*ThFac + OffsetY, 1.5*PhFac + OffsetX},  //5EF  //48
    { 0.360, 0.95,  0.410, 0.95, -0.17, 1.82, -0.135, 1.82, -5*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //-5A  //49
    { 0.360, 0.95,  0.410, 0.95, -0.09, 1.82, -0.055, 1.82, -5*ThFac + OffsetY, -1.0*PhFac + OffsetX },  //-5C  //50
    { 0.360, 0.95,  0.410, 0.95, -0.045, 1.8, -0.01, 1.8, -5*ThFac + OffsetY, 0.0*PhFac + OffsetX},  //-5D  //51
    { 0.360, 0.95,  0.410, 0.95, 0.0, 1.8, 0.03, 1.8, -5*ThFac + OffsetY, 1.0*PhFac + OffsetX},  //-5E  //52
    { 0.360, 0.95,  0.410, 0.95, 0.04, 1.85, 0.08, 1.85, -5*ThFac + OffsetY, 2.0*PhFac + OffsetX},  //-5F  //53
    { -0.280, 0.80, -0.240, 0.80,  -0.16, 1.6, -0.12, 1.6, 3*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //3A  //54
    { -0.360, 0.78, -0.310, 0.70, -0.15, 1.4, -0.11, 1.4, 4*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //4A  //55
    { -0.52, 0.84, -0.450, 0.84,  -0.124231-0.015, 1.371, -0.124231+0.015, 1.371, 6*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //6A   //56
    { -0.52, 0.84, -0.450, 0.84,  -0.0941722-0.015, 1.371, -0.0941722+0.015, 1.371, 6*ThFac + OffsetY, -2.0*PhFac + OffsetX },  //6B  //57
    { -0.52, 0.84, -0.450, 0.84,  -0.0573568-0.015, 1.371, -0.0573568+0.015, 1.371, 6*ThFac + OffsetY, -1.0*PhFac + OffsetX},  //6C  //58
    { -0.52, 0.84, -0.450, 0.84,  -0.0192785-0.015, 1.371, -0.0192785+0.015, 1.371, 6*ThFac + OffsetY, 0.0*PhFac + OffsetX},  //6D  //59
    { -0.52, 0.84, -0.450, 0.84,   0.0200637-0.015, 1.371, 0.0200637+0.015, 1.371, 6*ThFac + OffsetY, 1.0*PhFac + OffsetX},  //6E  //60
    { -0.52, 0.84, -0.450, 0.84,   0.0541045-0.015, 1.371, 0.0541045+0.015, 1.371, 6*ThFac + OffsetY, 2.0*PhFac + OffsetX},  //6F  //61
    { -0.52, 0.84, -0.450, 0.84,   0.0, 1.371, 0.0, 1.371, 6*ThFac + OffsetY, 3.0*PhFac + OffsetX},  //6G  //62
    { -0.430, 0.82, -0.390, 0.82, -0.13, 1.28, -0.1, 1.28, 5*ThFac + OffsetY, -3.0*PhFac + OffsetX},  //5A  //63
    { -0.430, 0.82, -0.390, 0.82, -0.09, 1.25, -0.07, 1.25, 5*ThFac + OffsetY, -2.0*PhFac + OffsetX},  //5B  //64
    {  0.450, 0.95,  0.500, 0.95, -0.1-0.025, 2.0, -0.1+0.013, 2.0, -6*ThFac + OffsetY, -2.0*PhFac + OffsetX},  //-6B  //65
    {  0.450, 0.95,  0.500, 0.95, -0.06-0.018, 2.0, -0.06+0.012, 2.0, -6*ThFac + OffsetY, -1.0*PhFac + OffsetX},  //-6C //66
    {  0.450, 0.95,  0.500, 0.95, -0.0226805-0.012, 2.0, -0.0226805+0.012, 2.0, -6*ThFac + OffsetY, 0.0*PhFac + OffsetX},  //-6D  /67
    {  0.450, 0.95,  0.500, 0.95,  0.02-0.012, 2.0, 0.02+0.012, 2.0, -6*ThFac + OffsetY, 1.0*PhFac + OffsetX},  //-6E  //68
    {  0.450, 0.95,  0.500, 0.95, -1, 2.221, +1, 2.221, -6*ThFac + OffsetY, 2.0*PhFac + OffsetX},  //-6F //69
    {  0.450, 0.95,  0.500, 0.95, -1, 2.221, +1, 2.221, -6*ThFac + OffsetY, -3.0*PhFac + OffsetX },  //-6A //70
  };  

      //*******************  TgY parameters **********************************

 double c0000, c0001, c0002, c0003, c0004, c0010, c0011, c0012, c0013, c0022, c0031, c0100, c0102, c0110, c0111, c1002, c1010, c1011, c1020;

  ifstream myfile3("ParameterResultsTgYSVD.dat");

  if (myfile3.is_open())
  {
    while (! myfile3.eof() )
    {
      myfile3 >>  c0000 >> c0001 >> c0002 >> c0003 >> c0004 >> c0010 >> c0011 >> c0012 >> c0013 >> c0022 >> c0031 >> c0100 >> c0102 >> c0110 >> c0111 >> c1002 >> c1010 >> c1011 >> c1020;
    }
    myfile3.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: c0000 = "<<c0000<<", c0001 = "<<c0001<<endl;


  //*************************************************************************




  for ( int k = 0; k<PN; k++ ){
  //for ( int k = 65; k<PN; k++ ){
  //for ( int k = 56; k<62; k++ ){
   
    ofstream outFiles;
    TString filename;
    filename.Form("outfile_%d_%s.dat",runnumber, PointNames[k].Data());
    outFiles.open(filename);
    

    TH2F *hFPXY = new TH2F("hFPXY","hFPXY", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThPh = new TH2F("hFPThPh","hFPThPh", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThX = new TH2F("hFPThX", "hFPThX", 500,-0.4,0.4,500,-0.8, 0.8);
    TH2F *hFPPhY = new TH2F("hFPPhY", "hFPPhY", 500,-0.15,0.15, 500,-0.2, 0.2);

    TH2F *hFPXYCut = new TH2F("hFPXYCut","hFPXYCut", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThPhCut = new TH2F("hFPThPhCut","hFPThPhCut", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThXCut = new TH2F("hFPThXCut", "hFPThXCut", 500,-0.4,0.4,500,-0.8, 0.8);
    TH2F *hFPPhYCut = new TH2F("hFPPhYCut", "hFPPhYCut", 500,-0.15,0.15, 500,-0.2, 0.2);
    TH1F *hDLt3 = new TH1F("hDLt3","hDLt3", 300, 700.0, 1000.0);

    int num = t1->GetEntries();
    cout<<"Number of entries : "<<num<<endl;
    if (MAXnum > 0 &&  MAXnum < num ) num = MAXnum;

    for (int i = 0; i<num; i++){
      t1->GetEntry(i);
      if ((i%1000==0) && i>0) cout<<i<<endl; 
    
      double x = BB_tr_x[0];
      double th = BB_tr_th[0];
      double y = BB_tr_y[0];
      double ph = BB_tr_ph[0];

      double TgY = c0000 + c0001*ph+ c0002*pow(ph,2) + c0003*pow(ph,3) + c0004*pow(ph,4) + c0010*y +  c0011*y*ph +  c0012*y*pow(ph,2) +  c0013*y*pow(ph,3) + c0022*pow(y,2)*pow(ph,2) + c0031*pow(y,3)*ph + c0100*th + c0102*th*pow(ph,2) +  c0110*th*y + c0111*th*y*ph + c1002*x*pow(ph,2) + c1010*x*y + c1011*x*y*ph+ c1020*x*pow(y,2);

      if ( DL_t3[0]< 780.0 || DL_t3[0] > 880.0 ) continue;
      if ( fabs(ReactPt_L_z)>0.20 ) continue;
      hDLt3->Fill(DL_t3[0]);

      TVector3 TCSX(0,-1,0);
      TVector3 TCSZ(TMath::Sin(scattering_angle),0,TMath::Cos(scattering_angle));
      TVector3 TCSY = TCSZ.Cross(TCSX);
      TRotation  fTCSInHCS;
      fTCSInHCS.RotateAxes(TCSX,TCSY,TCSZ);

      TVector3 BBBeamOffsetHCS( rb_x, rb_y, 0.0 );
      TVector3 BBBeamOffsetTCS=fTCSInHCS.Inverse()*(BBBeamOffsetHCS);
      TVector3 BBTgYVectorTCS(0.0, TgY, -TgY/tan(scattering_angle));
      TVector3 BBBeamSpotTCS = BBBeamOffsetTCS + BBTgYVectorTCS;

      TVector3 BBMissPointingHCS(OffsetX, OffsetY, OffsetZ);
      TVector3 BBMissPointingTCS = fTCSInHCS.Inverse()*(BBMissPointingHCS);
      
      TVector3 BBBeamSpotCorrectedTCS = BBBeamSpotTCS - BBMissPointingTCS; 
   

      TVector3 BeamSpotHCS( rb_x, rb_y, ReactPt_L_z );
      TVector3 BeamSpotTCS=fTCSInHCS.Inverse()*(BeamSpotHCS);
      double TargetY =  BeamSpotTCS.Y();
      if (fabs(TargetY - TgY) > 0.032) continue; 

      hFPXY->Fill(BB_tr_y[0],BB_tr_x[0]);
      hFPThPh->Fill(BB_tr_ph[0],BB_tr_th[0]);
      hFPThX->Fill(BB_tr_th[0],BB_tr_x[0]);
      hFPPhY->Fill(BB_tr_ph[0],BB_tr_y[0]);

      if (ApplyCuts( x, y, th, ph, FPCuts[k][1], FPCuts[k][0], FPCuts[k][3], FPCuts[k][2], FPCuts[k][5], FPCuts[k][4], FPCuts[k][7], FPCuts[k][6])){
        hFPXYCut->Fill(y,x);
        hFPThPhCut->Fill(ph,th);
        hFPThXCut->Fill(th,x);
        hFPPhYCut->Fill(ph,y);

        //outFiles<<BB_tr_x[0]<<"  "<<BB_tr_y[0]<<"  "<<BB_tr_th[0]<<"  "<<BB_tr_ph[0]<<"  "<<BB_gold_y<<"  "<<BBBeamSpotTCS.Y()<<"  "<<BBBeamSpotTCS.Z()<<endl;
        outFiles<<BB_tr_x[0]<<"  "<<BB_tr_y[0]<<"  "<<BB_tr_th[0]<<"  "<<BB_tr_ph[0]<<"  "<<BB_gold_y<<"  "<<BBBeamSpotCorrectedTCS.Y()<<"  "<<BBBeamSpotCorrectedTCS.Z()<<"  "<<FPCuts[k][9]<<endl;

      }


    }


    TCanvas* c0 = new TCanvas("c0","c0",660,660);
    c0->Divide(2,2,0.001,0.001);
    c0->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPXYCut->Draw("colz");
    c0->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThPhCut->Draw("colz");
    c0->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPPhYCut->Draw("colz");
    c0->cd(4);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThXCut->Draw("colz");

    TString FigureName;
    FigureName.Form("figure_%d_SieveCalibrationTgPh_%s.png",runnumber,PointNames[k].Data());
    c0->Print(FigureName.Data());


    TCanvas* c1 = new TCanvas("c1","c1",660,660);
    c1->Divide(2,1,0.001,0.001);
    c1->cd(1);
    gPad->SetGrid();
    hFPThX->Draw("");
    hFPThXCut->SetMarkerColor(kRed);
    hFPThXCut->Draw("same");
    c1->cd(2);
    gPad->SetGrid();
    hFPPhY->Draw("");
    hFPPhYCut->SetMarkerColor(kGreen);
    hFPPhYCut->Draw("same");

    FigureName.Form("figure_%d_SieveCalibrationTgPh2_%s.png", runnumber, PointNames[k].Data());
    c1->Print(FigureName.Data());

    TCanvas* c2 = new TCanvas("c2","c2",1170,660);
    c2->Divide(1,1,0.001,0.001);
    c2->cd(1);
    gPad->SetGrid();
    hDLt3->Draw();

    FigureName.Form("figure_%d_SieveCalibrationTgPh3_%s.png", runnumber, PointNames[k].Data());
    c2->Print(FigureName.Data());

    outFiles.close();

  }
  

  t1->Delete();
}

