#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph.h"
#include "TGraphErrors.h" 
#include "TROOT.h"
#include "TF1.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"
#include "TVector.h"

using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
        (maxarg1) : (maxarg2))

static int iminarg1,iminarg2;
#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
        (iminarg1) : (iminarg2))

float pythag(float a, float b)
{
	float absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+pow((absb/absa),2));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+pow((absa/absb),2)));
}

//void svdcmp(float **a, int m, int n, float w[], float **v)
void svdcmp(TMatrixF &a, int m, int n, TVectorF &w, TMatrixF &v)
{
	int flag,i,its,j,jj,k,l,nm;
	float anorm,c,f,g,h,s,scale,x,y,z;//,*rv;

	TVectorF rv1(1,n);
	g=scale=anorm=0.0;
	for (i=1;i<=n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<=m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<=m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=i;k<=m;k++) s += a[k][i]*a[k][j];
					f=s/h;
					for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
				}
				for (k=i;k<=m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<=n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<=n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<=n;k++) rv1[k]=a[i][k]/h;
				for (j=l;j<=m;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[j][k]*a[i][k];
					for (k=l;k<=n;k++) a[j][k] += s*rv1[k];
				}
				for (k=l;k<=n;k++) a[i][k] *= scale;
			}
		}
		anorm=FMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
        cout<<"burek"<<endl;
	for (i=n;i>=1;i--) {
		if (i < n) {
			if (g) {
				for (j=l;j<=n;j++)
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<=n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<=n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=IMIN(m,n);i>=1;i--) {
		l=i+1;
		g=w[i];
		for (j=l;j<=n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			for (j=l;j<=n;j++) {
				for (s=0.0,k=l;k<=m;k++) s += a[k][i]*a[k][j];
				f=(s/a[i][i])*g;
				for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
			}
			for (j=i;j<=m;j++) a[j][i] *= g;
		} else for (j=i;j<=m;j++) a[j][i]=0.0;
		++a[i][i];
	}
	for (k=n;k>=1;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=1;l--) {
				nm=l-1;
				if ((float)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((float)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((float)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=pythag(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=1;j<=m;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=1;j<=n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			if (its == 30) cerr<<"no convergence in 30 svdcmp iterations"<<endl;
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=pythag(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=pythag(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=1;jj<=n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=pythag(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=1;jj<=m;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
			//cout<<"Tle: "<<x<<",  "<<w[k]<<endl;
                        
		}
	}
	//free_vector(rv1,1,n);
}













void ExportDisplayData(TVectorF &b, TString filename, int NumberOfMatrixElements, int NumberOfPoints){
 
  ofstream outFile;
  outFile.open(filename.Data());

  double b0000 = b[1];
  double b0001 = b[2]; 
  double b0002 = b[3]; 
  double b0003 = b[4]; 
  double b0010 = b[5]; 
  double b0011 = b[6]; 
  double b0012 = b[7]; 
  double b0013 = b[8]; 
  double b0020 = b[9]; 
  double b0021 = b[10]; 
  double b0030 = b[11]; 
  double b0031 = b[12]; 
  double b0103 = b[13]; 
  double b0110 = b[14]; 
  double b0111 = b[15]; 
  double b0120 = b[16]; 
  double b0121 = b[17]; 
  double b0210 = b[18]; 
  double b1000 = b[19]; 
  double b1001 = b[20]; 
  double b1002 = b[21]; 
  double b1003 = b[22]; 
  double b1010 = b[23]; 
  double b1011 = b[24]; 
  double b1100 = b[25]; 
  double b1110 = b[26]; 
  double b1111 = b[27]; 
  double b1200 = b[28]; 
  double b1201 = b[29]; 
  double b1210 = b[30]; 
  double b2000 = b[31]; 
  double b2010 = b[32]; 
  double b2100 = b[33]; 
  double b2101 = b[34]; 
  double b2110 = b[35]; 
  double b2200 = b[36]; 
  double b3000 = b[37]; 
  double b3010 = b[38]; 
  double b3100 = b[39]; 
  double b4000 = b[40]; 
  
  double out[40] = { b0000, b0001, b0002, b0003, b0010, b0011, b0012, b0013, b0020, b0021, b0030 , b0031, b0103, b0110, b0111, b0120, b0121, b0210, b1000, b1001, b1002, b1003, b1010, b1011, b1100, b1110, b1111, b1200, b1201, b1210, b2000, b2010, b2100, b2101, b2110, b2200, b3000, b3010, b3100, b4000 };

  TString labels[40] = { "b0000", "b0001", "b0002", "b0003", "b0010", "b0011", "b0012", "b0013", "b0020", "b0021", "b0030", "b0031", "b0103", "b0110", "b0111", "b0120", "b0121", "b0210", "b1000", "b1001", "b1002", "b1003", "b1010", "b1011", "b1100", "b1110", "b1111", "b1200", "b1201", "b1210", "b2000", "b2010", "b2100", "b2101", "b2110", "b2200", "b3000", "b3010", "b3100", "b4000" };


  cout<<"Saving Vector: "<<endl;

  cout<<"N: "<<NumberOfPoints<<endl;
  outFile<<"N: "<<NumberOfPoints<<endl;

  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<labels[i]<<"  "<<out[i]<<" "<<endl;
     outFile<<labels[i]<<"  "<<out[i]<<" "<<endl;
  }
  outFile.close();
}

void ExportData(TVectorF &b, TString filename, int NumberOfMatrixElements = 40 ){
 
  ofstream outFile;
  outFile.open(filename.Data());


  double b0000 = b[1];
  double b0001 = b[2]; 
  double b0002 = b[3]; 
  double b0003 = b[4]; 
  double b0010 = b[5]; 
  double b0011 = b[6]; 
  double b0012 = b[7]; 
  double b0013 = b[8]; 
  double b0020 = b[9]; 
  double b0021 = b[10]; 
  double b0030 = b[11]; 
  double b0031 = b[12]; 
  double b0103 = b[13]; 
  double b0110 = b[14]; 
  double b0111 = b[15]; 
  double b0120 = b[16]; 
  double b0121 = b[17]; 
  double b0210 = b[18]; 
  double b1000 = b[19]; 
  double b1001 = b[20]; 
  double b1002 = b[21]; 
  double b1003 = b[22]; 
  double b1010 = b[23]; 
  double b1011 = b[24]; 
  double b1100 = b[25]; 
  double b1110 = b[26]; 
  double b1111 = b[27]; 
  double b1200 = b[28]; 
  double b1201 = b[29]; 
  double b1210 = b[30]; 
  double b2000 = b[31]; 
  double b2010 = b[32]; 
  double b2100 = b[33]; 
  double b2101 = b[34]; 
  double b2110 = b[35]; 
  double b2200 = b[36]; 
  double b3000 = b[37]; 
  double b3010 = b[38]; 
  double b3100 = b[39]; 
  double b4000 = b[40]; 
  
  double out[40] = { b0000, b0001, b0002, b0003, b0010, b0011, b0012, b0013, b0020, b0021, b0030 , b0031, b0103, b0110, b0111, b0120, b0121, b0210, b1000, b1001, b1002, b1003, b1010, b1011, b1100, b1110, b1111, b1200, b1201, b1210, b2000, b2010, b2100, b2101, b2110, b2200, b3000, b3010, b3100, b4000};

  cout<<"Saving Vector: "<<endl;
  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<out[i]<<"  "<<endl;
     outFile<<out[i]<<" ";
  }
  outFile<<endl;
  outFile.close();
}


int ImportData(TMatrixF &A0, TVectorF &bVec0, TString filename, int NMax=10000000, int NumberOfMatrixElements = 40){

  double DistanceToCollimator = 1.18;//1.26703;

  int n0 = 0;
  ifstream myfile0(filename.Data());
  if (myfile0.is_open())
  {
    while (! myfile0.eof() && n0<NMax )
    {
      n0++;
      float  x, y, th, ph, tgy, rby, rbz, yC;
      myfile0 >> x >> y >> th >> ph >> tgy >> rby >> rbz >> yC;

      A0.ResizeTo(A0.GetNrows()+1, NumberOfMatrixElements+1);
      bVec0.ResizeTo(bVec0.GetNrows()+1);
      A0[A0.GetNrows()-1][0] = 0;   
      A0[A0.GetNrows()-1][1] = 1 ;
      A0[A0.GetNrows()-1][2] = ph;
      A0[A0.GetNrows()-1][3] = pow(ph,2);
      A0[A0.GetNrows()-1][4] = pow(ph,3);
      A0[A0.GetNrows()-1][5] = y;
      A0[A0.GetNrows()-1][6] = y*ph;
      A0[A0.GetNrows()-1][7] = y*pow(ph,2);
      A0[A0.GetNrows()-1][8] = y*pow(ph,3);
      A0[A0.GetNrows()-1][9] = pow(y,2);
      A0[A0.GetNrows()-1][10] = pow(y,2)*ph;
      A0[A0.GetNrows()-1][11] = pow(y,3);
      A0[A0.GetNrows()-1][12] = pow(y,3)*ph;
      A0[A0.GetNrows()-1][13] = th*pow(ph,3);
      A0[A0.GetNrows()-1][14] = th*y;
      A0[A0.GetNrows()-1][15] = th*y*ph;
      A0[A0.GetNrows()-1][16] = th*pow(y,2);
      A0[A0.GetNrows()-1][17] = th*pow(y,2)*ph;
      A0[A0.GetNrows()-1][18] = pow(th,2)*y;
      A0[A0.GetNrows()-1][19] = x;
      A0[A0.GetNrows()-1][20] = x*ph;
      A0[A0.GetNrows()-1][21] = x*pow(ph,2);
      A0[A0.GetNrows()-1][22] = x*pow(ph,3);
      A0[A0.GetNrows()-1][23] = x*y;
      A0[A0.GetNrows()-1][24] = x*y*ph;
      A0[A0.GetNrows()-1][25] = x*th;
      A0[A0.GetNrows()-1][26] = x*th*y;
      A0[A0.GetNrows()-1][27] = x*y*th*ph;
      A0[A0.GetNrows()-1][28] = x*pow(th,2);
      A0[A0.GetNrows()-1][29] = x*pow(th,2)*ph;
      A0[A0.GetNrows()-1][30] = x*pow(th,2)*y;
      A0[A0.GetNrows()-1][31] = pow(x,2);
      A0[A0.GetNrows()-1][32] = pow(x,2)*y;
      A0[A0.GetNrows()-1][33] = pow(x,2)*th;
      A0[A0.GetNrows()-1][34] = pow(x,2)*th*ph;
      A0[A0.GetNrows()-1][35] = pow(x,2)*th*y;
      A0[A0.GetNrows()-1][36] = pow(x,2)*pow(th,2);
      A0[A0.GetNrows()-1][37] = pow(x,3);
      A0[A0.GetNrows()-1][38] = pow(x,3)*y;
      A0[A0.GetNrows()-1][39] = pow(x,3)*th;
      A0[A0.GetNrows()-1][40] = pow(x,4);

      bVec0[bVec0.GetNrows()-1] = atan2((yC - rby),(DistanceToCollimator - rbz));

      
    }
    myfile0.close();
    return n0;
  }
  else cout << "Unable to open file"; 
  return -1;

}





void TargetPhSVDAll(int runnumber=0)
{

  cout<<"Starting .... TargetThSVDAll"<<endl;
  const int NumberOfMatrixElements = 40;
  const int ConsideredMatrixElements = 40;
  const int NMax = 400; //400
  TMatrixF A(1,1);
  TVectorF b(1);
  
  int NumberOfElements[63];
  
  TString filename;
  filename.Form("outfile_m5B.dat", runnumber);
  NumberOfElements[0] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4A.dat", runnumber);
  NumberOfElements[1] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4B.dat", runnumber);
  NumberOfElements[2] = NumberOfElements[0] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4C.dat", runnumber);
  NumberOfElements[3] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4D.dat", runnumber);
  NumberOfElements[4] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4E.dat", runnumber);
  NumberOfElements[5] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m4F.dat", runnumber);
  NumberOfElements[6] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m3A.dat", runnumber);
  NumberOfElements[7] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m3D.dat", runnumber);
  NumberOfElements[8] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2A.dat", runnumber);
  NumberOfElements[9] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2B.dat", runnumber);
  NumberOfElements[10] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2C.dat", runnumber);
  NumberOfElements[11] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2D.dat", runnumber);
  NumberOfElements[12] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2E.dat", runnumber);
  NumberOfElements[13] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m2F.dat", runnumber);
  NumberOfElements[14] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1A.dat", runnumber);
  NumberOfElements[15] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1B.dat", runnumber);
  NumberOfElements[16] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1C.dat", runnumber);
  NumberOfElements[17] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1D.dat", runnumber);
  NumberOfElements[18] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1E.dat", runnumber);
  NumberOfElements[19] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m1F.dat", runnumber);
  NumberOfElements[20] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0A.dat", runnumber);
  NumberOfElements[21] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0B.dat", runnumber);
  NumberOfElements[22] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0C.dat", runnumber);
  NumberOfElements[23] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0D.dat", runnumber);
  NumberOfElements[24] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0E.dat", runnumber);
  NumberOfElements[25] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p0F.dat", runnumber);
  NumberOfElements[26] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p1A.dat", runnumber);
  NumberOfElements[27] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p1B.dat", runnumber);
  NumberOfElements[28] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p1C.dat", runnumber);
  NumberOfElements[29] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p1D.dat", runnumber);
  NumberOfElements[30] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);

  filename.Form("outfile_p1E.dat", runnumber);
  NumberOfElements[31] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p1F.dat", runnumber);
  NumberOfElements[32] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2A.dat", runnumber);
  NumberOfElements[33] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2B.dat", runnumber);
  NumberOfElements[34] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2C.dat", runnumber);
  NumberOfElements[35] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2D.dat", runnumber);
  NumberOfElements[36] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2E.dat", runnumber);
  NumberOfElements[37] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p2F.dat", runnumber);
  NumberOfElements[38] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p3B.dat", runnumber);
  NumberOfElements[39] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p3F.dat", runnumber);
  NumberOfElements[40] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  
  filename.Form("outfile_p4B.dat", runnumber);
  NumberOfElements[41] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p4C.dat", runnumber);
  NumberOfElements[42] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p4D.dat", runnumber);
  NumberOfElements[43] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p4E.dat", runnumber);
  NumberOfElements[44] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p4F.dat", runnumber);
  NumberOfElements[45] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p5C.dat", runnumber);
  NumberOfElements[46] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p5D.dat", runnumber);
  NumberOfElements[47] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m5A.dat", runnumber);
  NumberOfElements[48] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m5C.dat", runnumber);
  NumberOfElements[49] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);

  filename.Form("outfile_m5D.dat", runnumber);
  NumberOfElements[50] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m5E.dat", runnumber);
  NumberOfElements[51] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m5F.dat", runnumber);
  NumberOfElements[52] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p3A.dat", runnumber);
  NumberOfElements[53] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p4A.dat", runnumber);
  NumberOfElements[54] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p6B.dat", runnumber);
  NumberOfElements[55] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p6C.dat", runnumber);
  NumberOfElements[56] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);

  filename.Form("outfile_p6F.dat", runnumber);
  NumberOfElements[57] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p5A.dat", runnumber);
  NumberOfElements[58] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p5B.dat", runnumber);
  NumberOfElements[59] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  NumberOfElements[59] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m6B.dat", runnumber);
  NumberOfElements[60] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_m6C.dat", runnumber);
  NumberOfElements[61] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
  filename.Form("outfile_p6E.dat", runnumber);
  NumberOfElements[62] = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements);
 

  cout<<"Data Loaded!" <<endl;

  for (int i = 0; i<63; i++) cout<<"Element: "<<i<<" : "<<NumberOfElements[i]<<endl;
  //A.Print();
  //b.Print();

  TMatrixF V(A.GetNcols(), A.GetNcols());
  TVectorF w(A.GetNcols());

  TMatrixF U = A;
  svdcmp( U, U.GetNrows()-1, NumberOfMatrixElements, w, V);
  
  w.Print();
  
 
  // let's plot singular values
  double x[NumberOfMatrixElements+1];
  double y[NumberOfMatrixElements+1];
  double ey[NumberOfMatrixElements+1];
  double ex[NumberOfMatrixElements+1];
  for (int i = 1; i<=NumberOfMatrixElements; i++){
    x[i] = i;
    y[i] = w[i];
  }

  TCanvas *c1 = new TCanvas("c1","c1");
  c1->cd(1);
  gPad->SetLogy();
  gPad->SetGrid();
  TGraph *gr1 = new TGraph(NumberOfMatrixElements, x,y);
  gr1->SetMarkerStyle(22);
  gr1->SetMarkerColor(kRed);
  gr1->Draw("AP");
  

  TVectorF aVec(NumberOfMatrixElements+1);
  TVectorF errorVecSq(NumberOfMatrixElements+1);
  
  for(int k = 1; k<=ConsideredMatrixElements; k++ ){
      TVectorF UVec = TMatrixFColumn(U,k);
      TVectorF VVec = TMatrixFColumn(V,k);
      double sp = 0.0;
      for(int i = 1; i<=UVec.GetNrows()-1; i++) sp+= UVec[i]*b[i]; 
      

      for (int i = 1; i<=NumberOfMatrixElements; i++) aVec[i] = aVec[i] + sp/w[k]*VVec[i];
      for(int i = 1; i<=errorVecSq.GetNrows()-1; i++)errorVecSq[i] =  errorVecSq[i]+ pow(V[i][k]/w[k],2);

      x[k] = k;
      y[k] = aVec[7];     
      ex[k] = 0.0;
      ey[k] = sqrt(errorVecSq[7]) ;
  }

  errorVecSq.Print();

  TCanvas *c2 = new TCanvas("c2","c2");
  c2->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr2 = new TGraphErrors(NumberOfMatrixElements, x,y, ex, ey);
  gr2->SetMarkerStyle(23);
  gr2->SetMarkerColor(kRed);
  gr2->Draw("AP");
  

  TVectorD errorVec(NumberOfMatrixElements+1);
  for(int j = 1; j<=NumberOfMatrixElements+1; j++){
        errorVec[j] = sqrt(errorVecSq[j]);
   }
   errorVec.Print();
  //SVD.Solve(b);
 
  cout<<"Let's print out the vector y: "<<endl;
  for (int i = 1; i<=NumberOfMatrixElements; i++){
     cout<<aVec[i]<<" p/m  "<<errorVec[i]<<" or % "<<errorVec[i]/aVec[i]<<endl;
     //cout<<aVec[i]<<endl;
  }


 ExportData(aVec, "ParameterResultsTgPhSVD.dat", NumberOfMatrixElements);
 TString name;
 name.Form("ParameterResultsTgPhSVD_%d.dat", runnumber);
 ExportDisplayData(aVec, name.Data(), NumberOfMatrixElements, A.GetNrows());

}

