#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TProfile.h"
#include "TF1.h"


using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}

int ApplyCuts( double x, double y, double th, double ph, double Tk1, double Tn1, double Tk2, double Tn2, double Pk1, double Pn1, double Pk2, double Pn2 )
{


   if ( y > Pn1 + ph*Pk1 && y < Pn2 + ph*Pk2 && x > Tn1 + th*Tk1 && x < Tn2 + th*Tk2 ) return 1;
   else return 0;


}


void HRSLSieveReconstruction()
{
  HallA_style();

  //double distance2target = 2.0;
  double in2m = 0.0254;

  double offsetX = 0.0;
  //double offsetY = 0.0;//-0.015;


  Double_t PriKineLHe3_q_x;
  Double_t PriKineLHe3_q_y;
  Double_t PriKineLHe3_q_z;
  Double_t DL_t3;
  Double_t ReactPt_L_z;

  Double_t scattering_angle;

  const int PN = 48;

  double rCut = 0.02;

  double ThFac = 0.04925;
  double ThWidthFac = 0.00952;
  double PhFac = 0.03810;

  double FPCuts[PN][11] = {
    { 0.360, 0.95,  0.410, 0.95, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.13, 1.82, -0.095, 1.82, -2.0*PhFac + offsetX}, 
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.76, -0.125, 1.76, -3.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.73, -0.085, 1.73, -2.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.085, 1.73, -0.05, 1.73, -1.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.05, 1.73, -0.01, 1.73, -0.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.73, 0.03, 1.73, 1.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , -4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.73, 0.075, 1.73, 2.0*PhFac + offsetX},
    { 0.190, 0.95,  0.250, 0.95, -3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.7, -0.125, 1.7, -3.0*PhFac + offsetX},
    { 0.190, 0.95,  0.250, 0.95, -3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.045, 1.68, -0.01, 1.68, 0.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.64, -0.12, 1.64, -3.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.115, 1.64, -0.085, 1.64, -2.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.64, -0.045, 1.64, -1.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.045, 1.64, -0.01, 1.64, 0.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.64, 0.03, 1.64, 1.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.03, 1.64, 0.075, 1.64, 2.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.155, 1.62, -0.125, 1.62, -3.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.115, 1.62, -0.085, 1.62, -2.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.62, -0.045, 1.62, -1.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.62, -0.005, 1.62, 0.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.62, 0.03, 1.62, 1.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, -1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.62, 0.075, 1.62, 2.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.15, 1.6, -0.125, 1.6, -3.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.085, 1.6, -2.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.6, -0.045, 1.6, -1.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.6, 0.03, 1.6, 1.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.65, -0.125, 1.65, -3.0*PhFac + offsetX },
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.65, -0.085, 1.65, -2.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.6, -0.045, 1.6, -1.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.6, 0.03, 1.6, 1.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , 1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.155, 1.6, -0.125, 1.6, -3.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.085, 1.6, -2.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.58, -0.045, 1.58, -1.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.00, 1.6, 0.03, 1.6, 1.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.280, 0.80, -0.240, 0.80, 3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.08, 1.6, -2.0*PhFac + offsetX},
    { -0.280, 0.80, -0.240, 0.80 , 3*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.038, 1.6, 0.078, 1.6, 2.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.105, 1.4, -0.075, 1.4, -2.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.07, 1.4, -0.04, 1.4, -1.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.035, 1.4, -0.005, 1.4, 0.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, 4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.4, 0.03, 1.4, 1.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, 4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.4, 0.075, 1.4, 2.0*PhFac + offsetX},
    { -0.430, 0.82, -0.390, 0.82, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.06, 1.25, -0.04, 1.25, -1.0*PhFac + offsetX},
    { -0.430, 0.82, -0.390, 0.82, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.03, 1.25, -0.01, 1.25, 0.0*PhFac + offsetX}
};



  TString inFileName;
  inFileName.Form("./ROOTResults/eed_SieveCalibration_GoodEventsThCalibration_%d.root",runnumber);

  TChain *t1 =new TChain("tout");
  t1->Add(inFileName.Data());

  assert(t1);

  t1->SetBranchAddress( "BB.tr.tg_y", &BB_tr_tg_y);
  t1->SetBranchAddress( "BB.tr.tg_th", &BB_tr_tg_th);
  t1->SetBranchAddress( "BB.tr.tg_ph", &BB_tr_tg_ph);
  t1->SetBranchAddress( "BB.tr.tg_dp", &BB_tr_tg_dp);

  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "BB.tr.p", &BB_tr_p);

  t1->SetBranchAddress( "PriKineLHe3.q_x", &PriKineLHe3_q_x);
  t1->SetBranchAddress( "PriKineLHe3.q_y", &PriKineLHe3_q_y);
  t1->SetBranchAddress( "PriKineLHe3.q_z", &PriKineLHe3_q_z);
  
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  //t1->SetBranchAddress( "scattering_angle", &scattering_angle);

  scattering_angle = -90.5*3.141592654/180.0;


  TH2F *hColliXY = new TH2F("hColliXY","hColliXY", 500,-0.24,0.24,500,-0.4, 0.4);
  TH1F *hTgThCenterDifference = new TH1F("hTgThCenterDifference","hTgThCenterDifference", 300,-0.03,0.03);
  TH2F *hHRSLTgTh_BBTgTh = new TH2F("hHRSLTgTh_BBTgTh","hHRSLTgTh_BBTgTh", 300,-0.3,0.3, 300,-0.3, 0.3);
  TH1F *hTgThDifference = new TH1F("hTgThDifference","hTgThDifference", 300,-0.3,0.3);

  TH1F *hTargetYBB = new TH1F("hTargetYBB","hTargetYBB", 300,-0.35,0.35);
  TH1F *hTargetY = new TH1F("hTargetY","hTargetY", 300,-0.35,0.35);

  TH2F *hBBx_BBy = new TH2F("hBBx_BBy","hBBx_BBy", 300,-1.0,1.0,300,-0.3, 0.3);
  TH2F *hBBx_BBth= new TH2F("hBBx_BBth","hBBx_BBth", 300,-1.0,1.0,300,-0.3, 0.3);
  TH2F *hBBth_BBph= new TH2F("hBBth_BBph","hBBth_BBph", 300,-0.3,0.3,300,-0.3, 0.3);
  TH2F *hBBy_BBph= new TH2F("hBBy_BBph","hBBy_BBph", 300,-0.3,0.3,300,-0.3, 0.3);

  TH1F *hDLt3 = new TH1F("hDLt3","hDLt3", 300, 700.0, 1000.0);
  TH1F *hDLt3Cut = new TH1F("hDLt3Cut","hDLt3Cut", 300, 700.0, 1000.0);

  TH2F *hBBdp_BBTgTh = new TH2F("hBBdp_BBTgTh","hBBdp_BBTgTh", 300,-1.0,1.0, 300,-0.3, 0.3);
  TH2F *hBBdp_BBdeltaTgTh = new TH2F("hBBdp_BBdeltaTgTh","hBBdp_BBdeltaTgTh", 300,-0.5,1.0, 300,-0.03, 0.03);

    //*******************  Tg Th parameters **********************************

  double a0000, a0001, a0002, a0010, a0011, a0012, a0020, a0021, a0100, a0101, a0102, a0110, a0111, a0120, a0200, a0201, a0210, a1000, a1001, a1010, a1100, a1200, a2000, a2001, a2010, a2100, a2200, a3000, a3100, a4000;

  //ifstream myfile("ParameterResultsThMatlab.dat");
  ifstream myfile(TargetThMatrixFile.Data());
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >> a0000 >> a0001 >> a0002 >> a0010 >> a0011 >> a0012 >> a0020 >> a0021 >>  a0100 >> a0101 >> a0102 >> a0110 >> a0111 >> a0120 >> a0200 >> a0201 >> a0210 >> a1000 >> a1001 >> a1010 >> a1100 >> a1200 >> a2000 >> a2001 >> a2010 >> a2100 >> a2200 >> a3000 >> a3100 >> a4000;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<", a4000 = "<<a4000<<endl;



  double   b0000, b0001, b0002, b0003, b0004, b0011, b0012, b0013, b0020, b0021, b0022, b0030, b0031, b0040, b0101, b0102, b0103, b0110, b0111, b0112, b0121, b0130, b0201, b0210, b0211, b0220, b1000, b1001, b1002, b1003, b1010, b1011, b1012, b1020, b1021, b1030, b1100, b1101, b1102, b1110, b1111, b1200, b1201, b1210, b2000, b2001, b2002, b2010, b2011, b2020, b2100, b2101, b2110, b2200, b3000, b3001, b3010, b3100, b4001, b5001, b3101, b4101, b2201, b3201, b1103, b2103, b3020, b4020;

  ifstream myfile2 ("ParameterResultsPhiMatlab3.dat");
  if (myfile2.is_open())
  {
    while (! myfile2.eof() )
    {
      

       myfile2 >> b0000 >> b0001 >> b0002 >> b0003 >> b0004 >> b0011 >> b0012 >> b0013 >> b0020 >> b0021 >> b0022 >> b0030 >> b0031 >> b0040 >> b0101 >> b0102 >> b0103 >> b0110 >> b0111 >> b0112 >> b0121 >> b0130 >> b0201 >> b0210 >> b0211 >> b0220 >> b1000 >> b1001 >> b1002 >> b1003 >> b1010 >> b1011 >> b1012 >> b1020 >> b1021 >> b1030 >> b1100 >> b1101 >> b1102 >> b1110 >> b1111 >> b1200 >> b1201 >> b1210 >> b2000 >> b2001 >> b2002 >> b2010 >> b2011 >> b2020 >> b2100 >> b2101 >> b2110 >> b2200 >> b3000 >> b3001 >> b3010 >> b3100 >> b4001 >> b5001 >> b3101 >> b4101 >> b2201 >> b3201 >> b1103 >> b2103 >> b3020 >> b4020 ;

    }
    myfile2.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: b0000 = "<<b0000<<", b0001 = "<<b0001<<", b0002 = "<<b0002<<endl;


  //*************************************************************************

  // Target Y cut;

  double yCenter = -0.052;//0.01;
  double ySigma  = 0.015;
  

  // th-ph cut
  double xCut = 0.0;
  double yCut = 0.025;



  int num = t1->GetEntries();
   //num = 400000;
  cout<<"Number of entries : "<<num<<endl;

  for (int i = 0; i<num; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<endl; 

    double x = BB_tr_x;
    double th = BB_tr_th;
    double y = BB_tr_y;
    double ph = BB_tr_ph;

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);


    hDLt3->Fill(DL_t3);
    if ( fabs(DL_t3-802.5)>12.5 ) continue;
    hDLt3Cut->Fill(DL_t3);    


    double TargetY =  -ReactPt_L_z*sin(scattering_angle);
    hTargetY->Fill(TargetY);


   
    double ColliY = distance2target*tan(TgTh);
    double ColliX = BB_tr_tg_y + distance2target*tan(TgPh);
    
   
    double Lab_qx = PriKineLHe3_q_x;
    double Lab_qy = PriKineLHe3_q_y;
    double Lab_qz = PriKineLHe3_q_z;

    double BB_qz = Lab_qz*cos(scattering_angle) + Lab_qx*sin(scattering_angle);
    double BB_qy = -Lab_qz*sin(scattering_angle) + Lab_qx*cos(scattering_angle);
    double BB_qx = Lab_qy;

    double HRSLth = atan(BB_qx/BB_qz);

    hBBx_BBy->Fill(BB_tr_x, BB_tr_y );
    hBBx_BBth->Fill(BB_tr_x, BB_tr_th );
    hBBth_BBph->Fill(BB_tr_th, BB_tr_ph );
    hBBy_BBph->Fill(BB_tr_y, BB_tr_ph );
    hColliXY->Fill(ColliX,ColliY);
    hTargetYBB->Fill(BB_tr_tg_y);
    hHRSLTgTh_BBTgTh->Fill(HRSLth, TgTh);
    hTgThDifference->Fill(HRSLth - TgTh);
    hBBdp_BBTgTh->Fill(BB_tr_tg_dp, TgTh);

    for (int j = 0; j<PN; j++){
      if (ApplyCuts( x, y, th, ph, FPCuts[j][1], FPCuts[j][0], FPCuts[j][3], FPCuts[j][2], FPCuts[j][7], FPCuts[j][6], FPCuts[j][9], FPCuts[j][8])){
	 hTgThCenterDifference->Fill(atan(FPCuts[j][4]/distance2target) - TgTh );
         hBBdp_BBdeltaTgTh->Fill(BB_tr_tg_dp, atan(FPCuts[j][4]/distance2target) - TgTh );
      }
    } 

  }

  TCanvas* c0 = new TCanvas("c0","c0", 660,660);
  c0->Divide(1,1,0.001,0.001);
  c0->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hColliXY->Draw("colz");
  hColliXY->GetXaxis()->SetTitle("Collimator X [m]");
  hColliXY->GetYaxis()->SetTitle("Collimator Y [m]");
 
  TString figureName;
  figureName.Form("figure_%d_SieveCalibration_Bare.png",runnumber);
  c0->Print(figureName.Data());

  TLine *HorizontalLines[13];

  for (int i = 0; i<13; i++)
  { 
    HorizontalLines[i] = new TLine(-7.375*in2m + offsetX, (i-6)*1.938*in2m + offsetY, 7.375*in2m + offsetX, (i-6)*1.938*in2m + offsetY);
    HorizontalLines[i]->SetLineColor(kRed);
    HorizontalLines[i]->SetLineWidth(1.5);
    HorizontalLines[i]->Draw("same");
  }

  TLine *VerticalLines[7];

  for (int i = 0; i<7; i++)
  { 
    VerticalLines[i] = new TLine((i-3)*1.5*in2m + offsetX, -13.75*in2m + offsetY, (i-3)*1.5*in2m + offsetX, 13.75*in2m + offsetY);
    VerticalLines[i]->SetLineColor(kRed);
    VerticalLines[i]->SetLineWidth(1.5);
    VerticalLines[i]->Draw("same");
  }

  double Tocke[91][2];
  TEllipse *Holes[91];

  for (int i = 0; i<13; i++)
  { 
    for (int j = 0; j<7; j++)
    { 
      Tocke[j+ i*7][0] =  (j-3)*1.5*in2m + offsetX;
      Tocke[j+ i*7][1] =  (i-6)*1.938*in2m + offsetY;
      Holes[j+ i*7] = new TEllipse(Tocke[j+ i*7][0], Tocke[j+ i*7][1], 0.009525, 0.009525);
      Holes[j+ i*7]->SetLineColor(kRed);
      Holes[j+ i*7]->SetLineWidth(1);
      Holes[j+ i*7]->SetFillStyle(0);
      Holes[j+ i*7]->Draw();
    }
  }

  figureName.Form("figure_%d_SieveCalibration.png",runnumber);
  c0->Print(figureName.Data());


  TCanvas* c1 = new TCanvas("c1","c1");
  c1->Divide(1,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  hTgThCenterDifference->Draw("");
  figureName.Form("figure_%d_SieveCalibrationTgThCenterDiff.png",runnumber);
  c1->Print(figureName.Data());

  TCanvas* c2 = new TCanvas("c2","c2");
  c2->Divide(2,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  hTgThDifference->Draw("");
  c2->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgTh_BBTgTh->Draw("colz");
  figureName.Form("figure_%d_SieveCalibrationTgThDiff.png",runnumber);
  c2->Print(figureName.Data());

  TCanvas* c3 = new TCanvas("c3","c3");
  c3->Divide(2,1,0.001,0.001);
  c3->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgTh_BBTgTh->Draw("colz");
  c3->cd(2);
  gPad->SetGrid();
  TProfile *profyHRSLTgTh_BBTgTh = hHRSLTgTh_BBTgTh->ProfileY();
  profyHRSLTgTh_BBTgTh->Draw();
  TF1 *f1 = new TF1("fit1","1++x",-0.2,0.2);
  profyHRSLTgTh_BBTgTh->Fit("fit1","MR");
  figureName.Form("figure_%d_SieveCalibrationTgThFit.png",runnumber);
  c3->Print(figureName.Data());
  
  


  TCanvas* c5 = new TCanvas("c5","c5");
  c5->Divide(1,1,0.001,0.001);
  c5->cd(1);
  gPad->SetGrid();
  hTargetY->Draw();
  hTargetYBB->Draw("same");
  hTargetYBB->SetLineColor(kBlue);
  figureName.Form("figure_%d_SieveCalibrationTgY.png",runnumber);
  c5->Print(figureName.Data());


  TCanvas* c7 = new TCanvas("c7","c7");
  c7->Divide(1,1,0.001,0.001);
  c7->cd(1);
  gPad->SetGrid();
  hDLt3->Draw();
  hDLt3Cut->Draw("same");
  hDLt3Cut->SetLineColor(kBlue);
  
  figureName.Form("figure_%d_SieveCalibrationCoincTime.png",runnumber);
  c7->Print(figureName.Data());

  TCanvas* c8 = new TCanvas("c8","c8");
  c8->Divide(2,2,0.001,0.001);
  c8->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBx_BBy->Draw("colz");
  c8->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBx_BBth->Draw("colz");
  c8->cd(3);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBth_BBph->Draw("colz");
  c8->cd(4);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBy_BBph->Draw("colz");

  figureName.Form("figure_%d_SieveCalibrationFocalPlane.png",runnumber);
  c8->Print(figureName.Data());

  TCanvas* c9 = new TCanvas("c9","c9");
  c9->Divide(2,1,0.001,0.001);
  c9->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBdp_BBTgTh->Draw("colz");
  c9->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBdp_BBdeltaTgTh->Draw("colz");
  figureName.Form("figure_%d_SieveCalibrationTgThMomentum.png",runnumber);
  c9->Print(figureName.Data());





  t1->Delete();




}
