#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"


using namespace std;

int ApplyCuts(double x0, double y0, double r0, double x, double y)
{
  double radius = sqrt( pow(x0-x,2) + pow(y0-y,2));
   
  if (radius <= r0) return 1;
  else return 0;
}

void TargetThCalibration(int runnumber=0, double MAXnum=-1)
{

  double in2m = 0.0254;

  double OffsetX = 0.0; // this is stated in LABCS
  double OffsetY = 0.0; // this is stated in LABCS
  double OffsetZ = 0.0288454; // this is stated in LABCS

  double ThFac = 0.04925;

  Double_t BB_gold_y;
  Double_t BB_gold_th;
  Double_t BB_gold_ph;
  Double_t BB_gold_dp;
  Double_t BB_tr_x[10];
  Double_t BB_tr_y[10];
  Double_t BB_tr_th[10];
  Double_t BB_tr_ph[10];
  Double_t scattering_angle;
  Double_t rb_x;
  Double_t rb_y;
  Double_t ReactPt_L_z;
  Double_t DL_t3[100];

  


  TChain *t1 =new TChain("tout");

  TString name;
  name.Form("./ROOTResults/eed_Test_GoodEvents_%d.root", runnumber);
  t1->Add(name.Data());

  //t1->Add("./ROOTResults/eed_GoodEvents_3489.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3491.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3490.root");
 
  assert(t1);

  t1->SetBranchAddress( "BB.gold.y", &BB_gold_y);
  t1->SetBranchAddress( "BB.gold.th", &BB_gold_th);
  t1->SetBranchAddress( "BB.gold.ph", &BB_gold_ph);
  t1->SetBranchAddress( "BB.gold.dp", &BB_gold_dp);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "scattering_angle", &scattering_angle);
  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);
  t1->SetBranchAddress( "rb.x", &rb_x);
  t1->SetBranchAddress( "rb.y", &rb_y);
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  const int NSections = 13;

  TString PartialFilenames[NSections] = {
				"OutFile_New_ThLm6",
				"OutFile_New_ThLm5",
				"OutFile_New_ThLm4",
				"OutFile_New_ThLm3",
				"OutFile_New_ThLm2",
				"OutFile_New_ThLm1",
				"OutFile_New_ThL0",
				"OutFile_New_ThLp1",
				"OutFile_New_ThLp2",
				"OutFile_New_ThLp3",
				"OutFile_New_ThLp4",
				"OutFile_New_ThLp5",
				"OutFile_New_ThLp6"
				 };

  
  TString filenames[NSections];
  for (int m = 0; m<NSections; m++){
    filenames[m].Form("%s_%d.dat", PartialFilenames[m].Data(), runnumber);
  }

  cout<<"here I am!"<<endl;

   // be careful. CS is pointing downwards.
  double sieve_position[NSections] = { 
				   6.0* ThFac, 
				   5.0* ThFac, 
				   4.0* ThFac, 
				   3.0* ThFac, 
				   2.0* ThFac, 
				   1.0* ThFac, 
				   0.0* ThFac, 
				   -1.0* ThFac, 
				   -2.0* ThFac, 
				   -3.0* ThFac, 
				   -4.0* ThFac, 
				   -5.0* ThFac, 
				   -6.0* ThFac
				   };

  double FPCuts[NSections][4] = {
				{  0.450, 1.00,  0.500, 1.00 },
				{  0.360, 0.95,  0.410, 0.95 },
				{  0.275, 0.95,  0.330, 0.95 },
				{  0.190, 0.95,  0.250, 0.95 },
				{  0.110, 0.92,  0.160, 0.92 },
				{  0.025, 0.85,  0.075, 0.85 },
				{ -0.050, 0.85,  0.000, 0.80 },
				{ -0.125, 0.85, -0.080, 0.85 },
				{ -0.210, 0.82, -0.160, 0.82 },
				{ -0.280, 0.80, -0.240, 0.80 },
				{ -0.360, 0.78, -0.310, 0.70 },
				{ -0.430, 0.82, -0.390, 0.82 },
				{ -0.505, 0.81, -0.460, 0.81 } 
			     };
      
  //*******************  TgY parameters **********************************

  double a0000, a0001, a0002, a0003, a0004, a0010, a0011, a0012, a0013, a0022, a0031, a0100, a0102, a0110, a0111, a1002, a1010, a1011, a1020;

  //ifstream myfile("ParameterResultsTgYMatlabSVD.dat");
  ifstream myfile("ParameterResultsTgYSVD.dat");
  //ifstream myfile("ParameterResultsTgYSVD_2418_New.dat");
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >>  a0000 >> a0001 >> a0002 >> a0003 >> a0004 >> a0010 >> a0011 >> a0012 >> a0013 >> a0022 >> a0031 >> a0100 >> a0102 >> a0110 >> a0111 >> a1002 >> a1010 >> a1011 >> a1020;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<endl;

  //*************************************************************************


  for ( int k = 0; k<NSections; k++ ){
    ofstream outFiles;
    outFiles.open(filenames[k].Data());

    TH2F *hFPXY = new TH2F("hFPXY","hFPXY", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThPh = new TH2F("hFPThPh","hFPThPh", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThX = new TH2F("hFPThX", "hFPThX", 500,-0.4,0.4,500,-0.8, 0.8);
    TH2F *hFPPhY = new TH2F("hFPPhY", "hFPPhY", 500,-0.15,0.15, 500,-0.2, 0.2);

    TH2F *hFPXYCut = new TH2F("hFPXYCut","hFPXYCut", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThPhCut = new TH2F("hFPThPhCut","hFPThPhCut", 500,-0.4,0.4,500,-1.0, 1.0);
    TH2F *hFPThXCut = new TH2F("hFPThXCut", "hFPThXCut", 500,-0.4,0.4,500,-0.8, 0.8);
    TH2F *hFPPhYCut = new TH2F("hFPPhYCut", "hFPPhYCut", 500,-0.15,0.15, 500,-0.2, 0.2);
    TH1F *hDLt3 = new TH1F("hDLt3","hDLt3", 300, 700.0, 1000.0);

    int num = t1->GetEntries();
    //num = 100000;
    cout<<"Number of entries : "<<num<<endl;
   
    if (MAXnum > 0 &&  MAXnum < num ) num = MAXnum;


    for (int i = 0; i<num; i++){
      t1->GetEntry(i);
      if ((i%1000==0) && i>0) cout<<i<<endl; 

      double x = BB_tr_x[0];
      double th = BB_tr_th[0];
      double y = BB_tr_y[0];
      double ph = BB_tr_ph[0];

      double TgY =  a0000 + a0001*ph+ a0002*pow(ph,2) + a0003*pow(ph,3) + a0004*pow(ph,4) +  a0010*y +  a0011*y*ph +  a0012*y*pow(ph,2) +  a0013*y*pow(ph,3) + a0022*pow(y,2)*pow(ph,2) + a0031*pow(y,3)*ph + a0100*th + a0102*th*pow(ph,2) +  a0110*th*y + a0111*th*y*ph + a1002*x*pow(ph,2) + a1010*x*y + a1011*x*y*ph+ a1020*x*pow(y,2);

      //if ( fabs(DL_t3[0]-802.5)>12.5 ) continue;
      if ( DL_t3[0]< 780.0 || DL_t3[0] > 880.0 ) continue;
      if ( fabs(ReactPt_L_z)>0.20 ) continue;
      hDLt3->Fill(DL_t3[0]);

      TVector3 TCSX(0,-1,0);
      TVector3 TCSZ(TMath::Sin(scattering_angle),0,TMath::Cos(scattering_angle));
      TVector3 TCSY = TCSZ.Cross(TCSX);
      TRotation  fTCSInHCS;
      fTCSInHCS.RotateAxes(TCSX,TCSY,TCSZ);

      TVector3 BeamSpotHCS( rb_x, rb_y, ReactPt_L_z );
      TVector3 BeamSpotTCS=fTCSInHCS.Inverse()*(BeamSpotHCS);

      double TargetY =  BeamSpotTCS.Y();
      if (fabs(TargetY - TgY) > 0.032) continue;
 

      TVector3 BBBeamOffsetHCS( rb_x, rb_y, 0.0 );
      TVector3 BBBeamOffsetTCS=fTCSInHCS.Inverse()*(BBBeamOffsetHCS);
      TVector3 BBTgYVectorTCS(0.0, TgY, -TgY/tan(scattering_angle));
      TVector3 BBBeamSpotTCS = BBBeamOffsetTCS + BBTgYVectorTCS;

      TVector3 BBMissPointingHCS(OffsetX, OffsetY, OffsetZ);
      TVector3 BBMissPointingTCS = fTCSInHCS.Inverse()*(BBMissPointingHCS);
      
      TVector3 BBBeamSpotCorrectedTCS = BBBeamSpotTCS - BBMissPointingTCS; 

      
      hFPXY->Fill(BB_tr_y[0],BB_tr_x[0]);
      hFPThPh->Fill(BB_tr_ph[0],BB_tr_th[0]);
      hFPThX->Fill(BB_tr_th[0],BB_tr_x[0]);
      hFPPhY->Fill(BB_tr_ph[0],BB_tr_y[0]);

      if (BB_tr_x[0] > FPCuts[k][0] + BB_tr_th[0]*FPCuts[k][1] && BB_tr_x[0] < FPCuts[k][2] + BB_tr_th[0]*FPCuts[k][3] ){
        hFPXYCut->Fill(BB_tr_y[0],BB_tr_x[0]);
        hFPThPhCut->Fill(BB_tr_ph[0],BB_tr_th[0]);
        hFPThXCut->Fill(BB_tr_th[0],BB_tr_x[0]);
        hFPPhYCut->Fill(BB_tr_ph[0],BB_tr_y[0]);

        //outFiles<<BB_tr_x[0]<<"  "<<BB_tr_y[0]<<"  "<<BB_tr_th[0]<<"  "<<BB_tr_ph[0]<<"  "<<BB_gold_y<<"  "<<BBBeamSpotTCS.X()<<"  "<<BBBeamSpotTCS.Z()<<endl;
        outFiles<<BB_tr_x[0]<<"  "<<BB_tr_y[0]<<"  "<<BB_tr_th[0]<<"  "<<BB_tr_ph[0]<<"  "<<BB_gold_y<<"  "<<BBBeamSpotCorrectedTCS.X()<<"  "<<BBBeamSpotCorrectedTCS.Z()<<"  "<<sieve_position[k]<<endl;

      }


    }


    TCanvas* c0 = new TCanvas("c0","c0",660,660);
    c0->Divide(2,2,0.001,0.001);
    c0->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPXYCut->Draw("colz");
    c0->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThPhCut->Draw("colz");
    c0->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPPhYCut->Draw("colz");
    c0->cd(4);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThXCut->Draw("colz");

    TString FigureName;
    FigureName.Form("figure_%d_TargetThCalibration_Num%d.png",runnumber,k);
    c0->Print(FigureName.Data());


    TCanvas* c1 = new TCanvas("c1","c1",660,660);
    c1->Divide(1,1,0.001,0.001);
    c1->cd(1);
    gPad->SetGrid();
    hFPThX->Draw("");
    hFPThXCut->SetMarkerColor(kRed);
    hFPThXCut->Draw("same");
    FigureName.Form("figure_%d_TargetThCalibration2_Num%d.png", runnumber, k);
    c1->Print(FigureName.Data());


    TCanvas* c2 = new TCanvas("c2","c2",1170,660);
    c2->Divide(1,1,0.001,0.001);
    c2->cd(1);
    gPad->SetGrid();
    hDLt3->Draw();


  c1->Print("figure_TargetYCalibration_1.png");





    outFiles.close();

  }

  t1->Delete();
}

