#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TProfile.h"
#include "TF1.h"
#include "TVector3.h"
#include "TVector2.h"
#include "TRotation.h"
#include "TMath.h"


using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}

int ApplyCuts( double x, double y, double th, double ph, double Tk1, double Tn1, double Tk2, double Tn2, double Pk1, double Pn1, double Pk2, double Pn2 )
{


   if ( y > Pn1 + ph*Pk1 && y < Pn2 + ph*Pk2 && x > Tn1 + th*Tk1 && x < Tn2 + th*Tk2 ) return 1;
   else return 0;


}


void TargetThCalibrationResults(int runnumber=0)
{
  HallA_style();

  double in2m = 0.0254;
  double offsetX = 0.0;
  double offsetY = 0.0;

  double OffsetX = 0.0; // this is stated in LABCS
  double OffsetY = 0.0; // this is stated in LABCS
  double OffsetZ = 0.0288454; // this is stated in LABCS 

  double distance2target = 1.26703;

  Double_t BB_gold_y;
  Double_t BB_gold_th;
  Double_t BB_gold_ph;
  Double_t BB_gold_dp;
  Double_t BB_gold_p;
  Double_t BB_tr_x[10];
  Double_t BB_tr_y[10];
  Double_t BB_tr_th[10];
  Double_t BB_tr_ph[10];
  

  Double_t PriKineLHe3_q_x;
  Double_t PriKineLHe3_q_y;
  Double_t PriKineLHe3_q_z;
  Double_t DL_t3[100];
  Double_t ReactPt_L_z;
  Double_t rb_x;
  Double_t rb_y;

  Double_t scattering_angle;

  const int PN = 72;

  double rCut = 0.02;

  double ThFac = 0.04925;
  double ThWidthFac = 0.00952;
  double PhFac = 0.03810;

  double FPCuts[PN][11] = {
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.13, 1.82, -0.095, 1.82, -2.0*PhFac + offsetX}, 
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.76, -0.125, 1.76, -3.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.73, -0.085, 1.73, -2.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.085, 1.73, -0.05, 1.73, -1.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.05, 1.73, -0.01, 1.73, 0.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.73, 0.03, 1.73, 1.0*PhFac + offsetX},
    { 0.275, 0.95,  0.330, 0.95 , 4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.73, 0.075, 1.73, 2.0*PhFac + offsetX},
    { 0.190, 0.95,  0.250, 0.95, 3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.7, -0.125, 1.7, -3.0*PhFac + offsetX},
    { 0.190, 0.95,  0.250, 0.95, 3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.045, 1.68, -0.01, 1.68, 0.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.64, -0.12, 1.64, -3.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.115, 1.64, -0.085, 1.64, -2.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.64, -0.045, 1.64, -1.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.045, 1.64, -0.01, 1.64, 0.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.64, 0.03, 1.64, 1.0*PhFac + offsetX},
    { 0.110, 0.92,  0.160, 0.92, 2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.03, 1.64, 0.075, 1.64, 2.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.155, 1.62, -0.125, 1.62, -3.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.115, 1.62, -0.085, 1.62, -2.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.62, -0.045, 1.62, -1.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.62, -0.005, 1.62, 0.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.005, 1.62, 0.03, 1.62, 1.0*PhFac + offsetX},
    { 0.025, 0.85,  0.075, 0.85, 1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.62, 0.075, 1.62, 2.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.15, 1.6, -0.125, 1.6, -3.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.085, 1.6, -2.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.6, -0.045, 1.6, -1.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.6, 0.03, 1.6,   1.0*PhFac + offsetX},
    { -0.050, 0.85,  0.000, 0.80, 0*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.16, 1.65, -0.125, 1.65, -3.0*PhFac + offsetX },
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.65, -0.085, 1.65, -2.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.6, -0.045, 1.6, -1.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.6, 0.03, 1.6,   1.0*PhFac + offsetX},
    { -0.125, 0.85, -0.080, 0.85 , -1*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.155, 1.6, -0.125, 1.6, -3.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.085, 1.6, -2.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.08, 1.58, -0.045, 1.58, -1.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.04, 1.6, -0.005, 1.6, 0.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.00, 1.6, 0.03, 1.6,  1.0*PhFac + offsetX},
    { -0.210, 0.82, -0.160, 0.82, -2*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.6, 0.075, 1.6, 2.0*PhFac + offsetX},
    { -0.280, 0.80, -0.240, 0.80, -3*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.12, 1.6, -0.08, 1.6, -2.0*PhFac + offsetX},
    { -0.280, 0.80, -0.240, 0.80 , -3*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.038, 1.6, 0.078, 1.6, 2.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.105, 1.4, -0.075, 1.4, -2.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.07, 1.4, -0.04, 1.4, -1.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.035, 1.4, -0.005, 1.4, 0.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.4, 0.03, 1.4,    1.0*PhFac + offsetX},
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.035, 1.4, 0.075, 1.4, 2.0*PhFac + offsetX},
    { -0.430, 0.82, -0.390, 0.82, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.06, 1.25, -0.04, 1.25, -1.0*PhFac + offsetX},
    { -0.430, 0.82, -0.390, 0.82, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.03, 1.25, -0.01, 1.25, 0.0*PhFac + offsetX},
    { -0.430, 0.82, -0.390, 0.82, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.25, 0.03, 1.25, 1.5*PhFac + offsetX},  //5EF //48
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac,  -0.17, 1.82, -0.135, 1.82, -3.0*PhFac + offsetX},//-5A 
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac,  -0.09, 1.82, -0.055, 1.82, -1.0*PhFac + offsetX},  //-5C  
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.045, 1.8, -0.01, 1.8, 0.0*PhFac + offsetX},  //-5D  //51
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.0, 1.8, 0.03, 1.8, 1.0*PhFac + offsetX},  //-5E  //52
    { 0.360, 0.95,  0.410, 0.95, 5*ThFac + offsetY, sqrt(2)*ThWidthFac, 0.04, 1.85, 0.08, 1.85, 2.0*PhFac + offsetX},  //-5F  //53
    { -0.280, 0.80, -0.240, 0.80, -3*ThFac + offsetY, sqrt(2)*ThWidthFac,  -0.16, 1.6, -0.12, 1.6, -3.0*PhFac + offsetX},  //3A  
    { -0.360, 0.78, -0.310, 0.70, -4*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.15, 1.4, -0.11, 1.4, -3.0*PhFac + offsetX},  //4A  

    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.124231-0.015, 1.371, -0.124231+0.015, 1.371, -3.0*PhFac + offsetX},  //6A  
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.0941722-0.015, 1.371, -0.0941722+0.015, 1.371, -2.0*PhFac + offsetX},  //6B  
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.0573568-0.015, 1.371, -0.0573568+0.015, 1.371, -1.0*PhFac + offsetX},  //6C  
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.0192785-0.015, 1.371, -0.0192785+0.015, 1.371, 0.0*PhFac + offsetX},  //6D  
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac,  0.0200637-0.015, 1.371, 0.0200637+0.015, 1.371, 1.0*PhFac + offsetX},  //6E  //60
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac,  0.0541045-0.015, 1.371, 0.0541045+0.015, 1.371, 2.0*PhFac + offsetX},  //6F  //61
    { -0.52, 0.84, -0.450, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac,  0.0, 1.371, 0.0, 1.371, 3.0*PhFac + offsetX},  //6G  //62

    { -0.430, 0.82, -0.390, 0.82, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.13, 1.28, -0.1, 1.28, -3.0*PhFac + offsetX},  //5A  
    { -0.430, 0.82, -0.390, 0.82, -5*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.09, 1.25, -0.07, 1.25, -2.0*PhFac + offsetX},  //5B  

    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac,  -0.1-0.025, 2.0, -0.1+0.013,2.0, -2.0*PhFac + offsetX},  //-6B 
    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.06-0.018, 2.0, -0.06+0.012, 2.0, -1.0*PhFac + offsetX},  //-6C 
    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac, -0.0226805-0.012, 2.0, -0.0226805+0.012, 2.0, 0.0*PhFac + offsetX},  //-6D  
    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac,  0.02-0.012, 2.0, 0.02+0.012, 2.0, 1.0*PhFac + offsetX},  //-6E  
    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac,  -1, 2.221, +1, 2.221, 2.0*PhFac + offsetX},  //-6F
    {  0.450, 0.95,  0.500, 0.95, 6*ThFac + offsetY, sqrt(2)*ThWidthFac,  -1, 2.221, +1, 2.221, -3.0*PhFac + offsetX},  //-6A 
    {  -0.52, 0.84,  -0.45, 0.84, -6*ThFac + offsetY, sqrt(2)*ThWidthFac,  -1.0, 1.37, 1.0, 1.37, -3.0*PhFac + offsetX}  //All    
};



  //TString inFileName;
  //inFileName.Form("./ROOTResults/eed_GoodEvents_%d.root",runnumber);

  TChain *t1 =new TChain("tout");
  //t1->Add(inFileName.Data());
  
  //t1->Add("./ROOTResults/eed_GoodEvents_3489.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3490.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3491.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3493.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3495.root");

  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2618.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3488.root");
 
  //t1->Add("./ROOTResults/eed_GoodEvents_3496.root");

  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3265.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_2413.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_1518.root");
 // t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3608.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3151.root");

  assert(t1);

  t1->SetBranchAddress( "BB.gold.y", &BB_gold_y);
  t1->SetBranchAddress( "BB.gold.th", &BB_gold_th);
  t1->SetBranchAddress( "BB.gold.ph", &BB_gold_ph);
  t1->SetBranchAddress( "BB.gold.dp", &BB_gold_dp);
  t1->SetBranchAddress( "BB.gold.p", &BB_gold_p);

  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "rb.x", &rb_x);
  t1->SetBranchAddress( "rb.y", &rb_y);
  

  t1->SetBranchAddress( "PriKineLHe3.q_x", &PriKineLHe3_q_x);
  t1->SetBranchAddress( "PriKineLHe3.q_y", &PriKineLHe3_q_y);
  t1->SetBranchAddress( "PriKineLHe3.q_z", &PriKineLHe3_q_z);
  
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  t1->SetBranchAddress( "scattering_angle", &scattering_angle);


  TH2F *hColliXY = new TH2F("hColliXY","ColliXY - As we see it", 500,-0.24,0.24,500,-0.34, 0.34);
  TH2F *hColliXYOld = new TH2F("hColliXYOld","hColliXYOld", 500,-0.24,0.24,500,-0.34, 0.34);
  TH1F *hTgThCenterDifference = new TH1F("hTgThCenterDifference","hTgThCenterDifference", 300,-0.03,0.03);
  TH2F *hHRSLTgTh_BBTgTh = new TH2F("hHRSLTgTh_BBTgTh","hHRSLTgTh_BBTgTh", 300,-0.3,0.3, 300,-0.3, 0.3);
  TH2F *hHRSLTgTh_BBTgTh_Rotated = new TH2F("hHRSLTgTh_BBTgTh_Rotated","hHRSLTgTh_BBTgTh_Rotated", 300,-0.3,0.3, 300,-0.3, 0.3);

  TH1F *hTgThDifference = new TH1F("hTgThDifference","hTgThDifference", 300,-0.3,0.3);
  TH1F *hTargetYDifference = new TH1F("hTargetYDifference","hTargetYDifference", 300,-0.2,0.2);
 
  TH2F *hHRSLTgPh_BBTgPh = new TH2F("hHRSLTgPh_BBTgPh","hHRSLTgPh_BBTgPh", 300,-0.3,0.3, 300,-0.3, 0.3);
  TH1F *hTgPhDifference = new TH1F("hTgPhDifference","hTgPhDifference", 300,-0.3,0.3);

  TH1F *hTargetYBB = new TH1F("hTargetYBB","hTargetYBB", 300,-0.35,0.35);
  TH1F *hTargetY = new TH1F("hTargetY","hTargetY", 300,-0.35,0.35);

  TH2F *hBBx_BBy = new TH2F("hBBx_BBy","hBBx_BBy", 300,-1.0,1.0,300,-0.3, 0.3);
  TH2F *hBBx_BBth= new TH2F("hBBx_BBth","hBBx_BBth", 300,-1.0,1.0,300,-0.3, 0.3);
  TH2F *hBBth_BBph= new TH2F("hBBth_BBph","hBBth_BBph", 300,-0.3,0.3,300,-0.3, 0.3);
  TH2F *hBBy_BBph= new TH2F("hBBy_BBph","hBBy_BBph", 300,-0.3,0.3,300,-0.3, 0.3);

  TH1F *hDLt3 = new TH1F("hDLt3","hDLt3", 300, 700.0, 1000.0);
  TH1F *hDLt3Cut = new TH1F("hDLt3Cut","hDLt3Cut", 300, 700.0, 1000.0);

  TH2F *hBBdp_BBTgTh = new TH2F("hBBdp_BBTgTh","hBBdp_BBTgTh", 300,-1.0,1.0, 300,-0.3, 0.3);
  TH2F *hBBdp_BBdeltaTgTh = new TH2F("hBBdp_BBdeltaTgTh","hBBdp_BBdeltaTgTh", 300,-0.5,1.0, 300,-0.03, 0.03);

  TH2F *hColliXYCut[6];
  TH2F *hBBph_BByCut[6];
  TH2F *hColliXYCut2[6];
  TH2F *hBBph_BByCut2[6];

  for (int f= 0; f<6; f++){
    TString name;
    name.Form("hColliXYCut%d", 65+f);
    hColliXYCut[f] = new TH2F(name.Data(),name.Data(), 500,-0.15,0.15,500,-0.4, 0.4);
    name.Form("hBBph_BByCut%d", 65+f);
    hBBph_BByCut[f]= new TH2F(name.Data(),name.Data(), 300,-0.24,0.24,300,-0.2, 0.2);
    name.Form("hColliXYCut2%d", 65+f);
    hColliXYCut2[f] = new TH2F(name.Data(),name.Data(), 500,-0.15,0.15,500,-0.4, 0.4);
    name.Form("hBBph_BByCut2%d", 65+f);
    hBBph_BByCut2[f]= new TH2F(name.Data(),name.Data(), 300,-0.24,0.24,300,-0.2, 0.2);
  }

  TH2F *hBBph_BByCut3D= new TH2F("hBBph_BByCut3D","hBBph_BByCut3D", 100,-0.2,0.2,100,-0.4, 0.4);

    //*******************  Tg Th parameters **********************************

  double a0000, a0001, a0002, a0010, a0011, a0012, a0020, a0021, a0100, a0101, a0102, a0110, a0111, a0120, a0200, a0201, a0210, a1000, a1001, a1010, a1100, a1200, a2000, a2001, a2010, a2100, a2200, a3000, a3100, a4000;

  //ifstream myfile("ParameterResultsThMatlabFinal.dat");
  ifstream myfile("ParameterResultsThMatlabSVD.dat");
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >> a0000 >> a0001 >> a0002 >> a0010 >> a0011 >> a0012 >> a0020 >> a0021 >>  a0100 >> a0101 >> a0102 >> a0110 >> a0111 >> a0120 >> a0200 >> a0201 >> a0210 >> a1000 >> a1001 >> a1010 >> a1100 >> a1200 >> a2000 >> a2001 >> a2010 >> a2100 >> a2200 >> a3000 >> a3100 >> a4000;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<", a4000 = "<<a4000<<endl;

    //*******************  Tg Ph parameters **********************************
  double   b0000, b0001, b0002, b0003, b0004, b0011, b0012, b0013, b0020, b0021, b0022, b0030, b0031, b0040, b0101, b0102, b0103, b0110, b0111, b0112, b0121, b0130, b0201, b0210, b0211, b0220, b1000, b1001, b1002, b1003, b1010, b1011, b1012, b1020, b1021, b1030, b1100, b1101, b1102, b1110, b1111, b1200, b1201, b1210, b2000, b2001, b2002, b2010, b2011, b2020, b2100, b2101, b2110, b2200, b3000, b3001, b3010, b3100, b4001, b5001, b3101, b4101, b2201, b3201, b1103, b2103, b3020, b4020;

  ifstream myfile2 ("ParameterResultsPhiMatlabSVD.dat");
  if (myfile2.is_open())
  {
    while (! myfile2.eof() )
    {
      

       myfile2 >> b0000 >> b0001 >> b0002 >> b0003 >> b0004 >> b0011 >> b0012 >> b0013 >> b0020 >> b0021 >> b0022 >> b0030 >> b0031 >> b0040 >> b0101 >> b0102 >> b0103 >> b0110 >> b0111 >> b0112 >> b0121 >> b0130 >> b0201 >> b0210 >> b0211 >> b0220 >> b1000 >> b1001 >> b1002 >> b1003 >> b1010 >> b1011 >> b1012 >> b1020 >> b1021 >> b1030 >> b1100 >> b1101 >> b1102 >> b1110 >> b1111 >> b1200 >> b1201 >> b1210 >> b2000 >> b2001 >> b2002 >> b2010 >> b2011 >> b2020 >> b2100 >> b2101 >> b2110 >> b2200 >> b3000 >> b3001 >> b3010 >> b3100 >> b4001 >> b5001 >> b3101 >> b4101 >> b2201 >> b3201 >> b1103 >> b2103 >> b3020 >> b4020 ;

    }
    myfile2.close();
  }
  else cout << "Unable to open file"; 

  cout<<"Parameters: b0000 = "<<b0000<<", b0001 = "<<b0001<<", b0002 = "<<b0002<<endl;


      //*******************  TgY parameters **********************************

  double c0000, c0001, c0003, c0010, c0011, c0012, c0020, c0021, c0030, c0102, c0110, c0111, c0120, c0200, c0201, c0210, c1000, c1001, c1010, c1011, c1020, c1200, c2000, c2200, c3000;

  ifstream myfile3("ParameterResultsTgYMatlabSVD.dat");
  if (myfile3.is_open())
  {
    while (! myfile3.eof() )
    {
      myfile3 >>  c0000 >> c0001 >> c0003 >> c0010 >> c0011 >> c0012 >> c0020 >> c0021 >> c0030 >> c0102 >> c0110 >> c0111 >> c0120 >> c0200 >> c0201 >> c0210 >> c1000 >> c1001 >> c1010 >> c1011 >> c1020 >> c1200 >> c2000 >> c2200 >> c3000;
    }
    myfile3.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: c0000 = "<<c0000<<", c0001 = "<<c0001<<endl;


  //*************************************************************************

  // Target Y cut;

  double yCenter = -0.052;//0.01;
  double ySigma  = 0.015;
  
  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;

  for (int i = 0; i<num; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<endl; 

    scattering_angle = -75.0*3.141592654/180.0;
    TVector3 TCSX(0,-1,0);
    TVector3 TCSZ(TMath::Sin(scattering_angle),0,TMath::Cos(scattering_angle));
    TVector3 TCSY = TCSZ.Cross(TCSX);
    TRotation  fTCSInHCS;
    fTCSInHCS.RotateAxes(TCSX,TCSY,TCSZ);

    TVector3 BBMissPointingHCS(OffsetX, OffsetY, OffsetZ);
    TVector3 BBMissPointingTCS = fTCSInHCS.Inverse()*(BBMissPointingHCS);
    offsetX = -BBMissPointingTCS.X();
    offsetY = -BBMissPointingTCS.Y();

    double x = BB_tr_x[0];
    double th = BB_tr_th[0];
    double y = BB_tr_y[0];
    double ph = BB_tr_ph[0];

    double TgPh = b0000 + b0001*ph + b0101*ph*th + b1000*x + b1001*ph*x + b1100*th*x + b1101*ph*th*x + b0011*ph*y + b0110*th*y + b0111*ph*th*y + b1010*x*y + b1011*ph*x*y + b1110*th*x*y + b1111*ph*th*x*y + b0002*pow(ph,2) + b0102*th*pow(ph,2) + b1002*x*pow(ph,2) + b1102*th*x*pow(ph,2) + b0012*y*pow(ph,2) + b0112*th*y*pow(ph,2) + b1012*x*y*pow(ph,2) + b0003*pow(ph,3) + b0103*th*pow(ph,3) + b1003*x*pow(ph,3) + b0013*y*pow(ph,3) + b0004*pow(ph,4) + b0201*ph*pow(th,2) + b1200*x*pow(th,2) + b1201*ph*x*pow(th,2) + b0210*y*pow(th,2) + b0211*ph*y*pow(th,2) + b1210*x*y*pow(th,2) +  b2000*pow(x,2) + b2001*ph*pow(x,2) + b2100*th*pow(x,2) + b2101*ph*th*pow(x,2) + b2010*y*pow(x,2) + b2011*ph*y*pow(x,2) + b2110*th*y*pow(x,2) + b2002*pow(ph,2)*pow(x,2) + b2200*pow(th,2)*pow(x,2) + b3000*pow(x,3) + b3001*ph*pow(x,3) + b3100*th*pow(x,3) + b3010*y*pow(x,3) + b0020*pow(y,2) + b0021*ph*pow(y,2) + b0121*ph*th*pow(y,2) + b1020*x*pow(y,2) + b1021*ph*x*pow(y,2) + b0022*pow(ph,2)*pow(y,2) + b0220*pow(th,2)*pow(y,2) + b2020*pow(x,2)*pow(y,2) + b0030*pow(y,3) + b0031*ph*pow(y,3) + b0130*th*pow(y,3) + b1030*x*pow(y,3) + b0040*pow(y,4) + b1103*pow(ph,3)*th*x + b2103*pow(ph,3)*th*pow(x,2) + b2201*ph*pow(th,2)*pow(x,2) + b3101*ph*th*pow(x,3) + b3201*ph*pow(th,2)*pow(x,3) + b4001*ph*pow(x,4) + b4101*ph*th*pow(x,4) + b5001*ph*pow(x,5) + b3020*pow(x,3)*pow(y,2) + b4020*pow(x,4)*pow(y,2);

    double TgTh = a0000 + a0001*ph + a0002*pow(ph,2) + a0100*th + a0101*ph*th + a0102*pow(ph,2)*th + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1000*x + a1001*ph*x + a1100*th*x + a1200*pow(th,2)*x  + a2000*pow(x,2) + a2001*ph*pow(x,2) +  a2100*th*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) +  a3100*th*pow(x,3) + a4000*pow(x,4) + a0010*y + a0011*ph*y + a0012*pow(ph,2)*y + a0110*th*y + a0111*ph*th*y + a0210*pow(th,2)*y + a1010*x*y + a2010*pow(x,2)*y + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2);

    double TgY = c0000 + c0001*ph + c1000*x + c1001*ph*x + c0010*y + c0011*ph*y + c0110*th*y + c0111*ph*th*y + c1010*x*y + c1011*ph*x*y + c0102*th*pow(ph,2) + c0012*y*pow(ph,2) + c0003*pow(ph,3) + c0200*pow(th,2) + c0201*ph*pow(th,2) + c1200*x*pow(th,2) + c0210*y*pow(th,2) +  c2000*pow(x,2) + c2200*pow(th,2)*pow(x,2) + c3000*pow(x,3) + c0020*pow(y,2) + c0021*ph*pow(y,2) + c0120*th*pow(y,2) + c1020*x*pow(y,2) + c0030*pow(y,3);

    hDLt3->Fill(DL_t3[0]);
    //if ( fabs(DL_t3[0]-802.5)>12.5 ) continue;
    if ( fabs(DL_t3[0]-830.0)>50.0 ) continue; ////  !!!!!!!!!!!!!!!!!!!!!!!!!!!
    if ( fabs(ReactPt_L_z)>0.20 ) continue; // general cut on the cell
    if ( fabs(ReactPt_L_z)>0.15 ) continue; // more restrictive cut on gas
   
   // if ( DL_t3[0] < 825.0 || DL_t3[0] >840.0 ) continue;

    hDLt3Cut->Fill(DL_t3[0]);    


    TVector3 BeamSpotHCS( rb_x, rb_y, ReactPt_L_z );
    TVector3 BeamSpotTCS=fTCSInHCS.Inverse()*(BeamSpotHCS);

    TVector3 BBBeamOffsetHCS( rb_x, rb_y, 0.0 );
    TVector3 BBBeamOffsetTCS=fTCSInHCS.Inverse()*(BBBeamOffsetHCS);

    TVector3 BBTgYVectorTCS(0.0, TgY, -TgY/tan(scattering_angle));

    TVector3 BBBeamSpotTCS = BBBeamOffsetTCS + BBTgYVectorTCS;
    hTargetY->Fill(BBBeamSpotTCS.Y());

    TVector3 MomDirTCS(tan(TgTh), tan(TgPh),1);
    MomDirTCS *= (distance2target - BBBeamSpotTCS.Z()); // now we set proper lenght in TCS

    TVector3 ColliPosTCS = MomDirTCS  + BBBeamSpotTCS;

    TVector3 qVectorHCS(PriKineLHe3_q_x, PriKineLHe3_q_y, PriKineLHe3_q_z);
    TVector3 qVectorTCS = fTCSInHCS.Inverse()*(qVectorHCS);
    double HRSLth = atan(qVectorTCS.X()/qVectorTCS.Z());
    double HRSLph = atan(qVectorTCS.Y()/qVectorTCS.Z());
    double TargetY =  BeamSpotTCS.Y();

    if (fabs(TargetY - TgY) > 0.04) continue; 

    TVector2 ThetaVector(HRSLth, TgTh);
    TVector2 ThetaVectorRotated = ThetaVector.Rotate(-45.0*3.141592654/180.0);

    TVector2 PhiVector(HRSLph, TgPh);

    hHRSLTgTh_BBTgTh->Fill(ThetaVector.X(), ThetaVector.Y());
    hHRSLTgTh_BBTgTh_Rotated->Fill(ThetaVectorRotated.X(), ThetaVectorRotated.Y());
    hHRSLTgPh_BBTgPh->Fill(PhiVector.X(), PhiVector.Y());

    hBBx_BBy->Fill(x, y);
    hBBx_BBth->Fill(x, th );
    hBBth_BBph->Fill(th, ph );
    hBBy_BBph->Fill(y, ph );
    hColliXY->Fill(-ColliPosTCS.Y(),-ColliPosTCS.X());
    hColliXYOld->Fill(-1.0*(distance2target*tan(TgPh) + TgY),-1.0*distance2target*tan(TgTh));
    hTargetYBB->Fill(BB_gold_y);
    hTgThDifference->Fill(HRSLth - TgTh);
    hTgPhDifference->Fill(HRSLph - TgPh);
    hBBdp_BBTgTh->Fill(BB_gold_dp, TgTh);
    hTargetYDifference->Fill(TargetY-TgY);

    hBBph_BByCut3D->Fill(-ColliPosTCS.Y(),-ColliPosTCS.X());


    

    
    for (int f = 0; f<6; f++){

      for (int j = 71; j<72; j++){
        if (ApplyCuts( x, y, th, ph, FPCuts[j][1], FPCuts[j][0], FPCuts[j][3], FPCuts[j][2], FPCuts[j][7], FPCuts[j][6], FPCuts[j][9], FPCuts[j][8])){
	   TVector3 SieveHolePosTCS(FPCuts[j][4], FPCuts[j][10], distance2target);
           hColliXYCut[f]->Fill(-ColliPosTCS.Y(),-ColliPosTCS.X());
	   hBBph_BByCut[f]->Fill(ph, y );
           
        }
      } 

      for (int j = 56+f; j<57+f; j++){
        if (ApplyCuts( x, y, th, ph, FPCuts[j][1], FPCuts[j][0], FPCuts[j][3], FPCuts[j][2], FPCuts[j][7], FPCuts[j][6],   FPCuts[j][9], FPCuts[j][8])){
	   TVector3 SieveHolePosTCS(FPCuts[j][4], FPCuts[j][10], distance2target);
           hColliXYCut2[f]->Fill(-ColliPosTCS.Y(),-ColliPosTCS.X());
	   hBBph_BByCut2[f]->Fill(ph, y );
        }
      }
    }

    

  }

  TCanvas* c0 = new TCanvas("c0","c0", 660,660);
  c0->Divide(2,1,0.001,0.001);
  c0->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hColliXYOld->Draw("colz");
  hColliXYOld->GetXaxis()->SetTitle("Collimator -Y [m]");
  hColliXYOld->GetYaxis()->SetTitle("Collimator -X [m]");
 
  c0->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hColliXY->Draw("colz");
  hColliXY->GetXaxis()->SetTitle("Collimator -Y [m]");
  hColliXY->GetYaxis()->SetTitle("Collimator -X [m]");
 
  TString figureName;
  figureName.Form("figure_%d_SieveCalibration_Bare.png",runnumber);
  c0->Print(figureName.Data());

  TLine *HorizontalLines[13];


  

  for (int i = 0; i<13; i++)
  { 
    HorizontalLines[i] = new TLine(-7.375*in2m + offsetY, (i-6)*1.938*in2m + offsetX, 7.375*in2m + offsetY, (i-6)*1.938*in2m + offsetX);
    HorizontalLines[i]->SetLineColor(kRed);
    HorizontalLines[i]->SetLineWidth(1.5);
    HorizontalLines[i]->Draw("same");
  }

  TLine *VerticalLines[7];

  for (int i = 0; i<7; i++)
  { 
    VerticalLines[i] = new TLine((i-3)*1.5*in2m + offsetY, -13.75*in2m + offsetX, (i-3)*1.5*in2m + offsetY, 13.75*in2m + offsetX);
    VerticalLines[i]->SetLineColor(kRed);
    VerticalLines[i]->SetLineWidth(1.5);
    VerticalLines[i]->Draw("same");
  }

  double Tocke[91][2];
  TEllipse *Holes[91];

  for (int i = 0; i<13; i++)
  { 
    for (int j = 0; j<7; j++)
    { 
      Tocke[j+ i*7][0] =  (j-3)*1.5*in2m + offsetY;
      Tocke[j+ i*7][1] =  (i-6)*1.938*in2m + offsetX;
      Holes[j+ i*7] = new TEllipse(Tocke[j+ i*7][0], Tocke[j+ i*7][1], 0.009525, 0.009525);
      Holes[j+ i*7]->SetLineColor(kRed);
      Holes[j+ i*7]->SetLineWidth(1);
      Holes[j+ i*7]->SetFillStyle(0);
      Holes[j+ i*7]->Draw();
    }
  }

  figureName.Form("figure_%d_SieveCalibration.png",runnumber);
  c0->Print(figureName.Data());


  TCanvas* c1 = new TCanvas("c1","c1");
  c1->Divide(1,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  hTgThCenterDifference->Draw("");
  figureName.Form("figure_%d_SieveCalibrationTgThCenterDiff.png",runnumber);
  c1->Print(figureName.Data());

  TCanvas* c2 = new TCanvas("c2","c2");
  c2->Divide(2,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  hTgThDifference->Draw("");
  c2->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgTh_BBTgTh->Draw("colz");
  figureName.Form("figure_%d_SieveCalibrationTgThDiff.png",runnumber);
  c2->Print(figureName.Data());

  TCanvas* c3 = new TCanvas("c3","c3");
  c3->Divide(2,2,0.001,0.001);
  c3->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgTh_BBTgTh->Draw("colz");
  c3->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgTh_BBTgTh_Rotated->Draw("colz");
  c3->cd(3);
  gPad->SetGrid();
  TProfile *profyHRSLTgTh_BBTgTh = hHRSLTgTh_BBTgTh_Rotated->ProfileX();
  profyHRSLTgTh_BBTgTh->Draw();
  TF1 *f1 = new TF1("fit1","1++x",-0.2,0.2);
  profyHRSLTgTh_BBTgTh->Fit("fit1","MR");
  figureName.Form("figure_%d_SieveCalibrationTgThFit.png",runnumber);
  c3->Print(figureName.Data());
  
  TCanvas* c4 = new TCanvas("c4","c4");
  c4->Divide(2,1,0.001,0.001);
  c4->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hHRSLTgPh_BBTgPh->Draw("colz");
  c4->cd(2);
  gPad->SetGrid();
  hTgPhDifference->Draw();
  figureName.Form("figure_%d_SieveCalibrationTgPh.png",runnumber);
  c4->Print(figureName.Data()); 


  TCanvas* c5 = new TCanvas("c5","c5");
  c5->Divide(1,1,0.001,0.001);
  c5->cd(1);
  gPad->SetGrid();
  hTargetY->Draw();
  hTargetYBB->Draw("same");
  hTargetYBB->SetLineColor(kBlue);
  figureName.Form("figure_%d_SieveCalibrationTgY.png",runnumber);
  c5->Print(figureName.Data());


  TCanvas* c7 = new TCanvas("c7","c7");
  c7->Divide(1,1,0.001,0.001);
  c7->cd(1);
  gPad->SetGrid();
  hDLt3->Draw();
  hDLt3Cut->Draw("same");
  hDLt3Cut->SetLineColor(kBlue);
  
  figureName.Form("figure_%d_SieveCalibrationCoincTime.png",runnumber);
  c7->Print(figureName.Data());

  TCanvas* c8 = new TCanvas("c8","c8");
  c8->Divide(2,2,0.001,0.001);
  c8->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBx_BBy->Draw("colz");
  c8->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBx_BBth->Draw("colz");
  c8->cd(3);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBth_BBph->Draw("colz");
  c8->cd(4);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBy_BBph->Draw("colz");

  figureName.Form("figure_%d_SieveCalibrationFocalPlane.png",runnumber);
  c8->Print(figureName.Data());

  TCanvas* c9 = new TCanvas("c9","c9");
  c9->Divide(2,1,0.001,0.001);
  c9->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBdp_BBTgTh->Draw("colz");
  c9->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBdp_BBdeltaTgTh->Draw("colz");
  figureName.Form("figure_%d_SieveCalibrationTgThMomentum.png",runnumber);
  c9->Print(figureName.Data());

  TCanvas* c10[6];
  for (int f=0; f<6; f++){
    TString name;
    name.Form("c10%d",f);
    c10[f] = new TCanvas(name.Data(),name.Data());
    c10[f]->Divide(2,1,0.001,0.001);
    c10[f]->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hBBph_BByCut[f]->Draw("colz");
    hBBph_BByCut2[f]->Draw("same");
    hBBph_BByCut2[f]->SetMarkerColor(kRed);
    c10[f]->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hColliXYCut[f]->Draw();
    hColliXYCut2[f]->Draw("same");
    hColliXYCut2[f]->SetMarkerColor(kRed);
  }

  TCanvas* c11 = new TCanvas("c11","c11");
  c11->Divide(1,1,0.001,0.001);
  c11->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hBBph_BByCut3D->Draw("surf1");



  TCanvas* c12 = new TCanvas("c12","c12");
  c12->Divide(1,1,0.001,0.001);
  c12->cd(1);
  gPad->SetGrid();
  hTargetYDifference->Draw("");

  TCanvas* c13 = new TCanvas("c13","c13",450,900);
  c13->Divide(1,1,0.001,0.001);
  c13->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hColliXY->Draw("colz");
  hColliXY->GetXaxis()->SetTitle("Collimator -Y [m]");
  hColliXY->GetYaxis()->SetTitle("Collimator -X [m]");

  figureName.Form("figure_%d_Sieve.png",runnumber);
  c13->Print(figureName.Data());


  t1->Delete();




}
