#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"


using namespace std;

int ApplyCuts(double x0, double r0, double x)
{
  double radius = sqrt(pow(x - x0,2));
   
  if (radius <= r0) return 1;
  else return 0;
}

void TargetYCalibration( int runnumber)
{
  const int PN = 8;

  ofstream outFiles[PN];
  
  for (int i = 0; i<PN; i++)
  {
    TString name;
    name.Form("OutFileTargetY%d_%d.dat",i, runnumber);
    outFiles[i].open(name.Data());
  }

  double in2m = 0.0254;

  double offsetX = 0.0;
  double offsetY = 0.0;//-0.015;

  Double_t BB_gold_y;
  Double_t BB_gold_th;
  Double_t BB_gold_ph;
  Double_t BB_gold_dp;
  Double_t BB_tr_x[10];
  Double_t BB_tr_y[10];
  Double_t BB_tr_th[10];
  Double_t BB_tr_ph[10];
  Double_t DL_t3[100];

  Double_t ReactPt_L_z;
  Double_t rb_x;
  Double_t rb_y;
  Double_t scattering_angle;



  TChain *t1 =new TChain("tout");
  /*
  TString name;
  name.Form("./ROOTResults/eed_Test_GoodEvents_%d.root", runnumber);
  t1->Add(name.Data());
*/

  t1->Add("./ROOTResults/eed_Test_GoodEvents_2418.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2480.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2607.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3398.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3399.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3571.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3570.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3495.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3371.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3370.root");
  

  
  assert(t1);

  t1->SetBranchAddress( "BB.gold.y", &BB_gold_y);
  t1->SetBranchAddress( "BB.gold.th", &BB_gold_th);
  t1->SetBranchAddress( "BB.gold.ph", &BB_gold_ph);
  t1->SetBranchAddress( "BB.gold.dp", &BB_gold_dp);
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);
  t1->SetBranchAddress( "rb.x", &rb_x);
  t1->SetBranchAddress( "rb.y", &rb_y);
  t1->SetBranchAddress( "scattering_angle", &scattering_angle);


  TH1F *hTargetY = new TH1F("hTargetY","hTargetY", 300,-0.35,0.35);
  TH1F *hTargetYTrue = new TH1F("hTargetYTrue","hTargetYTrue", 300,-0.35,0.35);
  TH1F *hTargetYCenter = new TH1F("hTargetYCenter","hTargetYCenter", 300,-0.35,0.35);
  TH1F *hTargetYBB = new TH1F("hTargetYBB","hTargetYBB", 300,-0.35,0.35);
  TH1F *hDLt3 = new TH1F("hDLt3","hDLt3", 300, 700.0, 1000.0);
  TH1F *hDLt3Cut = new TH1F("hDLt3Cut","hDLt3Cut", 300, 700.0, 1000.0);

  TH2F *hFPXY = new TH2F("hFPXY","hFPXY", 500,-0.4,0.4,500,-1.0, 1.0);
  TH2F *hFPThPh = new TH2F("hFPThPh","hFPThPh", 500,-0.4,0.4,500,-1.0, 1.0);
  TH2F *hFPThX = new TH2F("hFPThX", "hFPThX", 500,-0.4,0.4,500,-0.8, 0.8);
  TH2F *hFPPhY = new TH2F("hFPPhY", "hFPPhY", 500,-0.15,0.15, 500,-0.2, 0.2);


  TH1F *hTargetYCut[PN];  
  TH2F *hFPXYCut[PN];
  TH2F *hFPThPhCut[PN];
  TH2F *hFPThXCut[PN];
  TH2F *hFPPhYCut[PN];
  TH2F *hFPPhTgYCut[PN];
  TH2F *hFPYTgYCut[PN];


  for(int i = 0; i<PN; i++)
  { 
    TString name;
    name.Form("TargetYCut_%d",i);  
    hTargetYCut[i] = new TH1F(name.Data(),name.Data(), 500, -0.35, 0.35 );
      
    name.Form("hFPXYCut_%d",i);  
    hFPXYCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.4,0.4,500,-1.0, 1.0);

    name.Form("hFPThPhCut_%d",i);  
    hFPThPhCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.4,0.4,500,-1.0, 1.0);

    name.Form("hFPThXCut_%d",i);  
    hFPThXCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.4,0.4,500,-1.0, 1.0);

    name.Form("hFPPhYCut_%d",i); 
    hFPPhYCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.2,0.2,500,-0.3, 0.3);

    name.Form("hFPPhTgYCut_%d",i); 
    hFPPhTgYCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.4,0.4,500,-0.3, 0.3);

    name.Form("hFPYTgYCut_%d",i); 
    hFPYTgYCut[i] = new TH2F(name.Data(),name.Data(), 500,-0.4,0.4,500,-0.3, 0.3);

  }

  
  const double rCut = 0.01;//0.008*2.35;



  double CarbonFoils[PN] = {
				(0.00119 + 3.0*0.066548),  //#0
                                (0.00119 + 2.0*0.066548), //#1
				(0.00119 + 1.0*0.066548), //#2
			     	(0.00119 - 0.0*0.066548), //#3
			     	(0.00119 - 1.0*0.066548), //#4
				(0.00119 - 2.0*0.066548), //#5
				(0.00119 - 3.0*0.066548 - 0.014), //#6
				(-0.291696) //#7
			    };

  double FPCuts[PN][4] = {
				{2.76, 0.18, 2.76, 0.12}, //#0
				{2.76, 0.12, 2.76, 0.09}, //#1
				{2.76, 0.06, 2.76, 0.02}, //#2
				{2.76, -0.01, 2.76, -0.04},  //#3
				{2.76, -0.07, 2.76, -0.1}, //#4
				{2.76, -0.13, 2.76, -0.17}, //#5
				{2.76, -0.2, 2.76, -0.24}, //#6
				{2.76, -0.27, 2.76, -0.32}//#7
			 };


  double FPPhCuts[PN][2] = {
				{-0.12, -0.09},  //#0
				{-0.09, -0.01}, //#1
				{-0.07, 0.03},  //#2
				{-0.06, 0.06},  //#3
				{-0.04, 0.09},  //#4
				{-0.0, 0.11},   //#5
				{0.06, 0.13},   //#6
				{0.11, 0.15}   //#7
			    };






  int num = t1->GetEntries();
  // num = 40000;
  cout<<"Number of entries : "<<num<<endl;

  for (int i = 0; i<num; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<endl; 
	
    TVector3 TCSX(0,-1,0);
    TVector3 TCSZ(TMath::Sin(scattering_angle),0,TMath::Cos(scattering_angle));
    TVector3 TCSY = TCSZ.Cross(TCSX);
    TRotation  fTCSInHCS;
    fTCSInHCS.RotateAxes(TCSX,TCSY,TCSZ);
   
   
    if ( DL_t3[0]< 780.0 || DL_t3[0] > 880.0 ) continue;
 
    hFPXY->Fill(BB_tr_y[0], BB_tr_x[0]);
    hFPThPh->Fill(BB_tr_ph[0], BB_tr_th[0]);
    hFPThX->Fill(BB_tr_th[0], BB_tr_x[0]);
    hFPPhY->Fill(BB_tr_ph[0], BB_tr_y[0]);
    

    for (int j = 0; j<PN; j++) // let's go over all the foils
    {
        if (fabs(ReactPt_L_z - CarbonFoils[j])> rCut) continue;

        if ((BB_tr_y[0] > FPCuts[j][0]*BB_tr_ph[0] + FPCuts[j][1] )  || (BB_tr_y[0] < FPCuts[j][2]*BB_tr_ph[0] + FPCuts[j][3] ))  continue;
	//if ( BB_tr_ph[0]< FPPhCuts[j][0] || BB_tr_ph[0] > FPPhCuts[j][1] ) continue;
        
        TVector3 BeamSpotHCS( rb_x, rb_y, ReactPt_L_z );
        TVector3 BeamSpotIdealHCS( 0.0, 0.0, ReactPt_L_z );
        TVector3 BeamSpotCenterHCS( rb_x, rb_y, CarbonFoils[j]);
        TVector3 BeamSpotTCS=fTCSInHCS.Inverse()*(BeamSpotHCS);
        TVector3 BeamSpotIdealTCS=fTCSInHCS.Inverse()*(BeamSpotIdealHCS);
        TVector3 BeamSpotCenterTCS=fTCSInHCS.Inverse()*(BeamSpotCenterHCS);

        double TargetY =  BeamSpotIdealTCS.Y();
        double TargetYTrue =  BeamSpotTCS.Y();
        double TargetYCenter =  BeamSpotCenterTCS.Y();
 
        hTargetY->Fill(TargetY);
        hTargetYTrue->Fill(TargetYTrue);
        hTargetYCenter->Fill(TargetYCenter);
        hDLt3->Fill(DL_t3[0]);

        hTargetYCut[j]->Fill(TargetYTrue);
        hFPXYCut[j]->Fill(BB_tr_y[0],BB_tr_x[0]);
        hFPThPhCut[j]->Fill(BB_tr_ph[0],BB_tr_th[0]);
        hFPThXCut[j]->Fill(BB_tr_th[0],BB_tr_x[0]);
        hFPPhYCut[j]->Fill(BB_tr_ph[0],BB_tr_y[0]);
        hFPPhTgYCut[j]->Fill(BB_tr_ph[0],TargetYTrue);
        hFPYTgYCut[j]->Fill(BB_tr_y[0],TargetYTrue);
        outFiles[j]<<BB_tr_x[0]<<"  "<<BB_tr_y[0]<<"  "<<BB_tr_th[0]<<"  "<<BB_tr_ph[0]<<"  "<<TargetYCenter<<"  "<<rCut<<"  "<<TargetYTrue<<endl;
  
    }   
  }


  TCanvas* c0 = new TCanvas("c0","c0",1200,800);
  c0->Divide(1,1,0.001,0.001);
  c0->cd(1);
  gPad->SetGrid();
  hTargetY->Draw();
  hTargetYTrue->Draw("same");
  hTargetYTrue->SetLineColor(kBlue);
  hTargetYCenter->Draw("same");
  hTargetYCenter->SetLineColor(kGreen);

  TLine *HorizontalLines[PN];

  for (int i = 0; i<PN; i++)
  { 
    HorizontalLines[i] = new TLine(-CarbonFoils[i]*sin(scattering_angle),0.0, -CarbonFoils[i]*sin(scattering_angle), 7000.0);
    HorizontalLines[i]->SetLineColor(kRed);
    HorizontalLines[i]->SetLineWidth(1.5);
    HorizontalLines[i]->Draw("same");
  }

  c0->Print("figure_TargetYCalibration_0.png");


  TCanvas* c1 = new TCanvas("c1","c1",1170,660);
  c1->Divide(1,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  //gStyle->SetPalette(1);
  hDLt3->Draw();


  c1->Print("figure_TargetYCalibration_1.png");



  TCanvas* cCut[PN];

  for(int i = 0; i<PN; i++)
  //for(int i = 5; i<6; i++)
  {
    TString name;
    name.Form("cCut%d",i);
    cCut[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    cCut[i]->Divide(3,3,0.001,0.001);
    cCut[i]->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hTargetY->Draw("");
    hTargetYCut[i]->Draw("same");
    hTargetYCut[i]->SetLineColor(kRed);
    cCut[i]->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hTargetYCut[i]->Draw();
    cCut[i]->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPXYCut[i]->Draw("colz");
    cCut[i]->cd(4);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThPhCut[i]->Draw("colz");
    cCut[i]->cd(5);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPPhYCut[i]->Draw("colz");
    cCut[i]->cd(6);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThXCut[i]->Draw("colz");
    cCut[i]->cd(7);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPPhTgYCut[i]->Draw("colz");
    cCut[i]->cd(8);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPYTgYCut[i]->Draw("colz");

    name.Form("figure_TargetYCalibratio_Cuts%d.png",i);
    cCut[i]->Print(name.Data());
  }



    TCanvas* c2 = new TCanvas("c2","c2",660,660);
    c2->Divide(2,2,0.001,0.001);
    c2->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPXY->Draw("colz");
    c2->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThPh->Draw("colz");
    c2->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPPhY->Draw("colz");
    c2->cd(4);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hFPThX->Draw("colz");

    TString FigureName;
    FigureName.Form("figure_TgYCalibrationFPPlots.png");
    c2->Print(FigureName.Data());



  t1->Delete();

  outFiles[0].close();



}

