#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TROOT.h"
#include "TF1.h"

using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}


int ApplyCuts(double x0, double r0, double x)
{
  double radius = sqrt(pow(x - x0,2));
   
  if (radius <= r0) return 1;
  else return 0;
}



Double_t fitfunction(Double_t *xx, Double_t *par)
{
  

  Double_t GBeO = par[0];
  Double_t xBeO = par[1];
  Double_t SigmaBeO = par[2];

  Double_t G1 = par[3];
  Double_t x1 = par[4];
  Double_t Sigma1 = par[5];

  Double_t G2 = par[6];
  Double_t x2 = par[7];
  Double_t Sigma2 = par[8];

  Double_t G3 = par[9];
  Double_t x3 = par[10];
  Double_t Sigma3 = par[11];

  Double_t G4 = par[12];
  Double_t x4 = par[13];
  Double_t Sigma4 = par[14];

  Double_t G5 = par[15];
  Double_t x5 = par[16];
  Double_t Sigma5 = par[17];

  Double_t G6 = par[18];
  Double_t x6 = par[19];
  Double_t Sigma6 = par[20];

  Double_t G7 = par[21];
  Double_t x7 = par[22];
  Double_t Sigma7 = par[23];


  double x =xx[0]; 

  double resultBeO = GBeO*exp(-pow((x-xBeO)/SigmaBeO,2)/2.0);
  double result1 = G1*exp(-pow((x-x1)/Sigma1,2)/2.0);
  double result2 = G2*exp(-pow((x-x2)/Sigma2,2)/2.0);
  double result3 = G3*exp(-pow((x-x3)/Sigma3,2)/2.0);
  double result4 = G4*exp(-pow((x-x4)/Sigma4,2)/2.0);
  double result5 = G5*exp(-pow((x-x5)/Sigma5,2)/2.0);
  double result6 = G6*exp(-pow((x-x6)/Sigma6,2)/2.0);
  double result7 = G7*exp(-pow((x-x7)/Sigma7,2)/2.0);





  return resultBeO + result1 + result2 + result3 + result4 + result5 + result6 + result7;

}


void TargetYCalibrationResults()
{

  HallA_style();


  const int PN = 8;

  Double_t BB_gold_y;
  Double_t BB_gold_th;
  Double_t BB_gold_ph;
  Double_t BB_gold_dp;
  Double_t BB_tr_x[10];
  Double_t BB_tr_y[10];
  Double_t BB_tr_th[10];
  Double_t BB_tr_ph[10];
  Double_t DL_t3[100];
  Double_t rb_x;
  Double_t rb_y;

  Double_t ReactPt_L_z;
  Double_t scattering_angle;



  TChain *t1 =new TChain("tout");


  t1->Add("./ROOTResults/eed_Test_GoodEvents_2617.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2616.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3483.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_2616.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_2617.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_2617.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3491.root");
  //t1->Add("./ROOTResults/eed_GoodEvents_3490.root");


  assert(t1);

  t1->SetBranchAddress( "BB.gold.y", &BB_gold_y);
  t1->SetBranchAddress( "BB.gold.th", &BB_gold_th);
  t1->SetBranchAddress( "BB.gold.ph", &BB_gold_ph);
  t1->SetBranchAddress( "BB.gold.dp", &BB_gold_dp);

  t1->SetBranchAddress( "BB.tr.x", &BB_tr_x);
  t1->SetBranchAddress( "BB.tr.y", &BB_tr_y);
  t1->SetBranchAddress( "BB.tr.th", &BB_tr_th);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "ReactPt_L.z", &ReactPt_L_z);
  t1->SetBranchAddress( "rb.x", &rb_x);
  t1->SetBranchAddress( "rb.y", &rb_y);
  t1->SetBranchAddress( "scattering_angle", &scattering_angle);
  t1->SetBranchAddress( "DL.t3", &DL_t3);

  TH1F *hTargetYBBNoCut = new TH1F("hTargetYBBNoCut","hTargetYBBNoCut", 300,-0.35,0.35);
  TH1F *hTargetYNoCut = new TH1F("hTargetYNoCut","hTargetYNoCut", 300,-0.35,0.35);

  TH1F *hTargetY = new TH1F("hTargetY","hTargetY", 300,-0.35,0.35);
  TH1F *hTargetYBB = new TH1F("hTargetYBB","hTargetYBB", 300,-0.35,0.35);
  TH1F *hTargetYBBfinal = new TH1F("hTargetYBBfinal","hTargetYBBfinal", 300,-0.35,0.35);
  
  TH1F *hTargetYBBDifference = new TH1F("hTargetYBBDifference","hTargetYBBDifference", 300,-0.2,0.2);

  TH2F *hTgYTgPh = new TH2F("hTgYTgPh","hTgYTgPh", 300,-0.4,0.4, 300,-0.4,0.4);
  TH2F *hTgYTgTh = new TH2F("hTgYTgTh","hTgYTgTh", 300,-0.4,0.4, 300,-0.4,0.4);
  TH2F *hTgYTgDp = new TH2F("hTgYTgDp","hTgYTgDp", 300,-0.4,0.4, 300,-0.6,1.0);
  TH2F *hTgYTgYHRSL = new TH2F("hTgYTgYHRSL","hTgYTgYHRSL", 300,-0.4,0.4, 300,-0.4,0.4);
  TH1F *hHRSLTgYResolution = new TH1F("hHRSLTgYResolution","hHRSLTgYResolution", 300,-0.2,0.2);

  TH1F *hDLt3 = new TH1F("hDLt3","hDLt3",300,700,1000);
  TH1F *hTargetYDifference = new TH1F("hTargetYDifference","hTargetYDifference", 300,-0.2,0.2);
  TH1F *hTgYBB = new TH1F("hTgYBB","hTgYBB", 200,-0.35,0.35);


  TH1F *hDLt3Background = new TH1F("hDLt3Background","hDLt3Background",300,700,1000);
  TH1F *hTargetYDifferenceBackground = new TH1F("hTargetYDifferenceBackground","hTargetYDifferenceBackground", 300,-0.2,0.2);
  

  TH1F *hTgYBBBackground = new TH1F("hTgYBBBackground","hTgYBBBackground", 200,-0.35,0.35);
  TH1F *hTgYBBPeak = new TH1F("hTgYBBPeak","hTgYBBPeak", 200,-0.35,0.35);
  TH1F *hTgYBBFinal = new TH1F("hTgYBBFinal","hTgYBBFinal", 200,-0.35,0.35);

  TH1F *hDLt3Peak = new TH1F("hDLt3Peak","hDLt3Peak",300,700,1000);
  TH1F *hTargetYDifferencePeak = new TH1F("hTargetYDifferencePeak","hTargetYDifferencePeak", 300,-0.2,0.2);
  

  TH1F *hBBTgYResolution = new TH1F("hBBTgYResolution","hBBTgYResolution", 300,-0.2,0.2);
  TH1F *hBBTgYResolutionBackground = new TH1F("hBBTgYResolutionBackground","hBBTgYResolutionBackground", 300,-0.2,0.2);
  TH1F *hBBTgYResolutionPeak = new TH1F("hBBTgYResolutionPeak","hBBTgYResolutionPeak", 300,-0.2,0.2);
  TH1F *hBBTgYResolutionFinal = new TH1F("hBBTgYResolutionFinal","hBBTgYResolutionFinal", 300,-0.2,0.2);


  
  
  const double rCut = 0.03;//0.008*2.35;



  TVector3 *CarbonFoils[PN];
  CarbonFoils[0] = new TVector3(0.0, 0.0, (0.00119 - 0.0*0.066548));
  CarbonFoils[1] = new TVector3(0.0, 0.0, (0.00119 - 1.0*0.066548)); //#5
  CarbonFoils[2] = new TVector3(0.0, 0.0, (0.00119 - 2.0*0.066548)); //#6
  CarbonFoils[3] = new TVector3(0.0, 0.0, (0.00119 - 3.0*0.066548 - 0.014)); //#7
  CarbonFoils[4] = new TVector3(0.0, 0.0, (-0.291696));
  CarbonFoils[5] = new TVector3(0.0, 0.0, (0.00119 + 1.0*0.066548)); //#3
  CarbonFoils[6] = new TVector3(0.0, 0.0, (0.00119 + 2.0*0.066548)); //#2
  CarbonFoils[7] = new TVector3(0.0, 0.0, (0.00119 + 3.0*0.066548));  //#1


  cout<<"Bla:::: "<<CarbonFoils[4]->Z()<<endl;

  double FPCuts[PN][4] = {
				{2.76, -0.01, 2.76, -0.04},     //#4
				{2.76, -0.07, 2.76, -0.1},  //#5
				{2.76, -0.13, 2.76, -0.17}, //#6
				{2.76, -0.2, 2.76, -0.24}, //#7
				{2.76, -0.27, 2.76, -0.32}, //#8
				{2.76, 0.06, 2.76, 0.02}, //#3
				{2.76, 0.12, 2.76, 0.09}, //#2
				{2.76, 0.18, 2.76, 0.12}  //#1
			 };

  double FPPhCuts[PN][2] = {
				{-0.06, 0.06},  //#4
				{-0.04, 0.09},  //#5
				{-0.0, 0.11},   //#6
				{0.06, 0.13},   //#7
				{0.11, 0.15},   //#8
				{-0.07, 0.03},  //#1
				{-0.09, -0.01}, //#2
				{-0.12, -0.09}  //#1
			    };



    //*******************  TgY parameters **********************************

  double a0000, a0001, a0003, a0010, a0011, a0012, a0020, a0021, a0030, a0102, a0110, a0111, a0120, a0200, a0201, a0210, a1000, a1001, a1010, a1011, a1020, a1200, a2000, a2200, a3000;

  ifstream myfile("ParameterResultsTgYMatlabSVD.dat");
  //ifstream myfile("ParameterResultsTgYSVD.dat");
  if (myfile.is_open())
  {
    while (! myfile.eof() )
    {
      myfile >>  a0000 >> a0001 >> a0003 >> a0010 >> a0011 >> a0012 >> a0020 >> a0021 >> a0030 >> a0102 >> a0110 >> a0111 >> a0120 >> a0200 >> a0201 >> a0210 >> a1000 >> a1001 >> a1010 >> a1011 >> a1020 >> a1200 >> a2000 >> a2200 >> a3000;
    }
    myfile.close();
  }
  else cout << "Unable to open file"; 



  cout<<"Parameters: a0000 = "<<a0000<<", a0001 = "<<a0001<<endl;


  //*************************************************************************
 
  scattering_angle = -75.0*3.141592654/180.0;

  TVector3 TCSX(0,-1,0);
  TVector3 TCSZ(TMath::Sin(scattering_angle),0,TMath::Cos(scattering_angle));
  TVector3 TCSY = TCSZ.Cross(TCSX);
  TRotation  fTCSInHCS;
  fTCSInHCS.RotateAxes(TCSX,TCSY,TCSZ);


  int num = t1->GetEntries();
  // num = 40000;
  cout<<"Number of entries : "<<num<<endl;

  for (int i = 0; i<num; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<endl; 






    double x = BB_tr_x[0];
    double th = BB_tr_th[0];
    double y = BB_tr_y[0];
    double ph = BB_tr_ph[0];


    double TgYBB = a0000 + a0001*ph + a1000*x + a1001*ph*x + a0010*y + a0011*ph*y + a0110*th*y + a0111*ph*th*y + a1010*x*y + a1011*ph*x*y + a0102*th*pow(ph,2) + a0012*y*pow(ph,2) + a0003*pow(ph,3) + a0200*pow(th,2) + a0201*ph*pow(th,2) + a1200*x*pow(th,2) + a0210*y*pow(th,2) +  a2000*pow(x,2) + a2200*pow(th,2)*pow(x,2) + a3000*pow(x,3) + a0020*pow(y,2) + a0021*ph*pow(y,2) + a0120*th*pow(y,2) + a1020*x*pow(y,2) + a0030*pow(y,3);


    TVector3 BeamSpotHCS( rb_x, rb_y, ReactPt_L_z );
    TVector3 BeamSpotTCS=fTCSInHCS.Inverse()*(BeamSpotHCS);
    double TargetY =  BeamSpotTCS.Y();


    if (fabs(TargetY - TgYBB)>0.04 ) continue;

    hDLt3->Fill(DL_t3[0]);
    hTargetYDifference->Fill(TargetY-TgYBB);
    hTgYBB->Fill(TgYBB);

 
    if (DL_t3[0]>760 && DL_t3[0]<780){
    //if (DL_t3[0]>880 && DL_t3[0]<900){
      hDLt3Background->Fill(DL_t3[0]);
      hTargetYDifferenceBackground->Fill(TargetY - TgYBB );
      hTgYBBBackground->Fill(TgYBB);
    }
    if (DL_t3[0]>780 && DL_t3[0]<880){
      hDLt3Peak->Fill(DL_t3[0]);
      hTargetYDifferencePeak->Fill(TargetY - TgYBB );
      hTgYBBPeak->Fill(TgYBB);
    }


    hTargetYNoCut->Fill(TargetY);
    hTargetYBBNoCut->Fill(TgYBB);




    for (int j = 0; j<PN; j++)
    {
        if (fabs(ReactPt_L_z - CarbonFoils[j]->Z())> rCut) continue;
        if ((BB_tr_y[0] > FPCuts[j][0]*BB_tr_ph[0] + FPCuts[j][1] )  || (BB_tr_y[0] < FPCuts[j][2]*BB_tr_ph[0] + FPCuts[j][3] ))  continue;
	//if ( BB_tr_ph[0]< FPPhCuts[j][0] || BB_tr_ph[0] > FPPhCuts[j][1] ) continue;

	TVector3 BeamSpotCenterHCS( rb_x, rb_y, CarbonFoils[j]->Z());
        TVector3 BeamSpotCenterTCS=fTCSInHCS.Inverse()*(BeamSpotCenterHCS);
        double TargetYCenter =  BeamSpotCenterTCS.Y();

        

        hTargetY->Fill(TargetY);
        hTargetYBB->Fill(TgYBB);
        hTargetYBBfinal->Fill(BB_gold_y);
        //hTargetYDifference->Fill(TargetY-TgYBB);
        hTargetYBBDifference->Fill(BB_gold_y-TgYBB);

        hTgYTgPh->Fill(TgYBB, BB_gold_ph);
        hTgYTgTh->Fill(TgYBB, BB_gold_th);
        hTgYTgDp->Fill(TgYBB, BB_gold_dp);
        hTgYTgYHRSL->Fill(TgYBB, TargetY);

        
        hBBTgYResolution->Fill(TargetYCenter - TgYBB );
        hHRSLTgYResolution->Fill(TargetYCenter - TargetY );


        if (DL_t3[0]>760 && DL_t3[0]<780){
          hBBTgYResolutionBackground->Fill(TargetYCenter - TgYBB );
        }

        if (DL_t3[0]>780 && DL_t3[0]<880){
          hBBTgYResolutionPeak->Fill(TargetYCenter - TgYBB );
        }

        

    }

  }

  TCanvas* c0 = new TCanvas("c0","c0",1200,800);
  c0->Divide(1,2,0.001,0.001);
  c0->cd(1);
  gPad->SetGrid();
  hTargetYNoCut->Draw();
  hTargetYBBNoCut->Draw("same");
  hTargetYBBNoCut->SetLineColor(kBlue);
  c0->cd(2);
  gPad->SetGrid();
  hTargetY->Draw();
  hTargetYBB->Draw("same");
  hTargetYBB->SetLineColor(kBlue);


  TLine *HorizontalLines[PN];

  for (int i = 0; i<PN; i++)
  { 
    TVector3 CF = fTCSInHCS.Inverse()*(*CarbonFoils[i]);
    double CFz = CF.Y();
    HorizontalLines[i] = new TLine(CFz,0.0, CFz, 7000.0);
    HorizontalLines[i]->SetLineColor(kRed);
    HorizontalLines[i]->SetLineWidth(1.5);
    HorizontalLines[i]->Draw("same");
  }

  c0->cd(2);
  gPad->SetGrid();
  hTargetYDifference->Draw();

  c0->Print("figure_TgYCalibrationResults.png");


  TCanvas* c1 = new TCanvas("c1","c1",1200,800);
  c1->Divide(2,2,0.001,0.001);
  c1->cd(1);
  gStyle->SetPalette(1);
  hTgYTgPh->Draw("colz");

  TLine *HorizontalLines1[PN];
  for (int i = 0; i<PN; i++)
  { 
    TVector3 CF = fTCSInHCS.Inverse()*(*CarbonFoils[i]);
    double CFz = CF.Y();
    HorizontalLines1[i] = new TLine(CFz,-1.0, CFz, 1.0);
    HorizontalLines1[i]->SetLineColor(kRed);
    HorizontalLines1[i]->SetLineWidth(1.5);
    HorizontalLines1[i]->Draw("same");
  }

  c1->cd(2);
  gStyle->SetPalette(1);
  hTgYTgTh->Draw("colz");

  TLine *HorizontalLines2[PN];
  for (int i = 0; i<PN; i++)
  { 
    TVector3 CF = fTCSInHCS.Inverse()*(*CarbonFoils[i]);
    double CFz = CF.Y();
    HorizontalLines2[i] = new TLine(CFz,-1.0, CFz, 1.0);
    HorizontalLines2[i]->SetLineColor(kRed);
    HorizontalLines2[i]->SetLineWidth(1.5);
    HorizontalLines2[i]->Draw("same");
  }



  c1->cd(3);
  gStyle->SetPalette(1);
  hTgYTgDp->Draw("colz");

  TLine *HorizontalLines3[PN];
  for (int i = 0; i<PN; i++)
  { 
    TVector3 CF = fTCSInHCS.Inverse()*(*CarbonFoils[i]);
    double CFz = CF.Y();
    HorizontalLines3[i] = new TLine(CFz,-1.0, CFz, 1.0);
    HorizontalLines3[i]->SetLineColor(kRed);
    HorizontalLines3[i]->SetLineWidth(1.5);
    HorizontalLines3[i]->Draw("same");
  }


  c1->cd(4);
  gStyle->SetPalette(1);
  hTgYTgYHRSL->Draw("colz");

  c1->Print("figure_TgYCalibrationResults1.png");


  TCanvas* c2 = new TCanvas("c2","c2",1200,800);
  c2->Divide(1,2,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  hBBTgYResolution->Draw();
  hBBTgYResolutionBackground->Draw("same");
  hBBTgYResolutionBackground->SetLineColor(kGreen);
  hBBTgYResolutionPeak->Draw("same");
  hBBTgYResolutionPeak->SetLineColor(kBlue);
  hBBTgYResolutionFinal->Add(hBBTgYResolutionPeak, 1);
  hBBTgYResolutionFinal->Add(hBBTgYResolutionBackground, -5);
  hBBTgYResolutionFinal->Draw("same");
  hBBTgYResolutionFinal->SetLineColor(kRed); 
  c2->cd(2);
  gPad->SetGrid();
  hHRSLTgYResolution->Draw();
  
  c2->Print("figure_TgYCalibrationResults2.png");

  TCanvas* c3 = new TCanvas("c3","c3",1200,800);
  c3->Divide(1,2,0.001,0.001);
  c3->cd(1);
  gPad->SetGrid();
  hTargetYBB->Draw();  
  hTargetYBBfinal->Draw("same");
  hTargetYBBfinal->SetLineColor(kRed);

  c3->cd(2);
  gPad->SetGrid();
  hTargetYBBDifference->Draw();


  c3->Print("figure_TgYCalibrationResults3.png");



  TH1F *hTargetYDifferenceFinal = new TH1F("hTargetYDifferenceFinal","hTargetYDifferenceFinal", 300,-0.2,0.2);

  TCanvas* c4 = new TCanvas("c4","c4",1170,660);
  c4->Divide(2,1,0.001,0.001);
  c4->cd(1);
  gPad->SetGrid();
  //gStyle->SetPalette(1);
  hDLt3->Draw();
  hDLt3Background->Draw("same");
  hDLt3Background->SetLineColor(kGreen);
  hDLt3Peak->Draw("same");
  hDLt3Peak->SetLineColor(kBlue);

  c4->cd(2);
  gPad->SetGrid();
  hTargetYDifference->Draw();
  hTargetYDifference->GetYaxis()->SetRangeUser(0,3000);
  hTargetYDifferenceBackground->Draw("same");
  hTargetYDifferenceBackground->SetLineColor(kGreen);
  hTargetYDifferencePeak->Draw("same");
  hTargetYDifferencePeak->SetLineColor(kBlue);
  hTargetYDifferenceFinal->Add(hTargetYDifferencePeak, 1);
  hTargetYDifferenceFinal->Add(hTargetYDifferenceBackground, -5);
  hTargetYDifferenceFinal->Draw("same");
  hTargetYDifferenceFinal->SetLineColor(kRed); 





  TCanvas* c5 = new TCanvas("c5","c5",1170,660);
  c5->Divide(2,1,0.001,0.001);
  c5->cd(1);
  gPad->SetGrid();
  //gStyle->SetPalette(1);
  hDLt3->Draw();
  hDLt3Background->Draw("same");
  hDLt3Background->SetLineColor(kGreen);
  hDLt3Peak->Draw("same");
  hDLt3Peak->SetLineColor(kBlue);

  c5->cd(2);
  gPad->SetGrid();
  hTgYBB->Draw();
  hTgYBBBackground->Draw("same");
  hTgYBBBackground->SetLineColor(kGreen);
  hTgYBBPeak->Draw("same");
  hTgYBBPeak->SetLineColor(kBlue);
  hTgYBBFinal->Add(hTgYBBPeak, 1);
  hTgYBBFinal->Add(hTgYBBBackground, -5);
  hTgYBBFinal->Draw("same");
  hTgYBBFinal->SetLineColor(kRed); 



  TCanvas* c6 = new TCanvas("c6","c6",1170,660);
  c6->Divide(1,1,0.001,0.001);
  c6->cd(1);
  gPad->SetGrid();
  hTgYBBFinal->Draw();
  hTgYBBFinal->SetLineColor(kRed);
  hTgYBBFinal->SetTitle("12C Carbon Target");
  hTgYBBFinal->GetXaxis()->SetTitle("TgY [m]");

  TF1 *f1 = new TF1("f1", fitfunction, -0.32, 0.32, 24);

  
  TVector3 CF[8];
  for (int i = 0; i<8; i++) CF[i] = fTCSInHCS.Inverse()*(*CarbonFoils[i]);

  f1->SetParLimits(0, 50.0, 200.0);
  f1->SetParLimits(1, CF[4].Y() - 0.01,  CF[4].Y() + 0.03);
  f1->FixParameter(2, 0.011);

  f1->SetParLimits(3, 100.0, 300.0);
  f1->SetParLimits(4, CF[3].Y() - 0.03, CF[3].Y() + 0.03);
  f1->FixParameter(5, 0.012);

  f1->SetParLimits(6, 300.0, 600.0);
  f1->SetParLimits(7, CF[2].Y() - 0.03, CF[2].Y() + 0.03);
  f1->FixParameter(8, 0.011);

  f1->SetParLimits(9, 300.0, 1000.0);
  f1->SetParLimits(10, CF[1].Y() - 0.03, CF[1].Y() + 0.03);
  f1->FixParameter(11, 0.012);
   //f1->SetParLimits(11, 0.01, 0.02);

  f1->SetParLimits(12, 300.0, 600.0);
  f1->SetParLimits(13, CF[0].Y() - 0.03, CF[0].Y() + 0.03);
  f1->FixParameter(14, 0.01099);
  //f1->SetParLimits(14, 0.009, 0.02);

  f1->SetParLimits(15, 300.0, 550.0);
  f1->SetParLimits(16, CF[5].Y() - 0.03, CF[5].Y() + 0.03);
  f1->FixParameter(17, 0.0122);

  f1->SetParLimits(18, 100.0, 300.0);
  f1->SetParLimits(19, CF[6].Y() - 0.03, CF[6].Y() + 0.04);
  f1->FixParameter(20, 0.013);

  f1->SetParLimits(21, 20.0, 100.0);
  f1->SetParLimits(22, CF[7].Y() - 0.01, CF[7].Y() + 0.01);
  f1->FixParameter(23, 0.02);
  
  hTgYBBFinal->Fit("f1", "RWWV");


  TF1 *fRes1 = new TF1("fRes1", "gaus", -0.32, 0.32);
  fRes1->SetParameter(0, f1->GetParameter(0));
  fRes1->SetParameter(1, f1->GetParameter(1));
  fRes1->SetParameter(2, f1->GetParameter(2));
  fRes1->SetLineColor(kGreen+2);
  fRes1->Draw("same");

  TF1 *fRes2 = new TF1("fRes2", "gaus", -0.32, 0.32);
  fRes2->SetParameter(0, f1->GetParameter(3));
  fRes2->SetParameter(1, f1->GetParameter(4));
  fRes2->SetParameter(2, f1->GetParameter(5));
  fRes2->SetLineColor(kGreen+2);
  fRes2->Draw("same");

  TF1 *fRes3 = new TF1("fRes3", "gaus", -0.32, 0.32);
  fRes3->SetParameter(0, f1->GetParameter(6));
  fRes3->SetParameter(1, f1->GetParameter(7));
  fRes3->SetParameter(2, f1->GetParameter(8));
  fRes3->SetLineColor(kGreen+2);
  fRes3->Draw("same");

  TF1 *fRes4 = new TF1("fRes4", "gaus", -0.32, 0.32);
  fRes4->SetParameter(0, f1->GetParameter(9));
  fRes4->SetParameter(1, f1->GetParameter(10));
  fRes4->SetParameter(2, f1->GetParameter(11));
  fRes4->SetLineColor(kGreen+2);
  fRes4->Draw("same");

  TF1 *fRes5 = new TF1("fRes5", "gaus", -0.32, 0.32);
  fRes5->SetParameter(0, f1->GetParameter(12));
  fRes5->SetParameter(1, f1->GetParameter(13));
  fRes5->SetParameter(2, f1->GetParameter(14));
  fRes5->SetLineColor(kGreen+2);
  fRes5->SetFillStyle(3003);
  fRes5->SetFillColor(kGreen+2);
  fRes5->Draw("same");

  TF1 *fRes6 = new TF1("fRes6", "gaus", -0.32, 0.32);
  fRes6->SetParameter(0, f1->GetParameter(15));
  fRes6->SetParameter(1, f1->GetParameter(16));
  fRes6->SetParameter(2, f1->GetParameter(17));
  fRes6->SetLineColor(kGreen+2);
  fRes6->Draw("same");

  TF1 *fRes7 = new TF1("fRes7", "gaus", -0.32, 0.32);
  fRes7->SetParameter(0, f1->GetParameter(18));
  fRes7->SetParameter(1, f1->GetParameter(19));
  fRes7->SetParameter(2, f1->GetParameter(20));
  fRes7->SetLineColor(kGreen+2);
  fRes7->Draw("same");

  TF1 *fRes8 = new TF1("fRes8", "gaus", -0.32, 0.32);
  fRes8->SetParameter(0, f1->GetParameter(21));
  fRes8->SetParameter(1, f1->GetParameter(22));
  fRes8->SetParameter(2, f1->GetParameter(23));
  fRes8->SetLineColor(kGreen+2);
  fRes8->Draw("same");
  f1->Draw("same");




 
  t1->Delete();

 



}

