#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph.h"
#include "TGraphErrors.h" 
#include "TROOT.h"
#include "TF1.h"
#include "TMatrixD.h"
#include "TMatrixTBaseD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"

using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}

void ExportDisplayData(TVectorD &b, TString filename, int NumberOfMatrixElements, int NumberOfPoints){
 
  ofstream outFile;
  outFile.open(filename.Data());


  double b0000 = b[0]; 
  double b0001 = b[1]; 
  double b0003 = b[2]; 
  double b0102 = b[3]; 
  double b0200 = b[4]; 
  double b0201 = b[5]; 
  double b1000 = b[6]; 
  double b1001 = b[7]; 
  double b1200 = b[8]; 
  double b2000 = b[9];
  double b2200 = b[10]; 
  double b3000 = b[11]; 
  double b0010 = b[12]; 
  double b0011 = b[13]; 
  double b0012 = b[14]; 
  double b0110 = b[15]; 
  double b0111 = b[16]; 
  double b0210 = b[17]; 
  double b1010 = b[18]; 
  double b1011 = b[19]; 
  double b0020 = b[20]; 
  double b0021 = b[21]; 
  double b0120 = b[22]; 
  double b1020 = b[23]; 
  double b0030 = b[24];

  double out[25] = { b0000, b0001, b0003, b0010, b0011, b0012, b0020, b0021,
          b0030, b0102, b0110, b0111, b0120, b0200, b0201, b0210, b1000, b1001, b1010, b1011, b1020,
          b1200, b2000, b2200, b3000};

  TString labels[25] = { "b0000", "b0001", "b0003", "b0010", "b0011", "b0012", "b0020", "b0021","b0030", "b0102", "b0110", "b0111", "b0120", "b0200", "b0201", "b0210", "b1000", "b1001", "b1010", "b1011", "b1020", "b1200", "b2000", "b2200", "b3000"};
  
  cout<<"Saving Vector: "<<endl;

  cout<<"N: "<<NumberOfPoints<<endl;
  outFile<<"N: "<<NumberOfPoints<<endl;

  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<labels[i]<<"  "<<out[i]<<" "<<endl;
     outFile<<labels[i]<<"  "<<out[i]<<" "<<endl;
  }
  outFile.close();
}

void ExportData(TVectorD &b, TString filename, int NumberOfMatrixElements = 25 ){
 
  ofstream outFile;
  outFile.open(filename.Data());


  double b0000 = b[0]; 
  double b0001 = b[1]; 
  double b0003 = b[2]; 
  double b0102 = b[3]; 
  double b0200 = b[4]; 
  double b0201 = b[5]; 
  double b1000 = b[6]; 
  double b1001 = b[7]; 
  double b1200 = b[8]; 
  double b2000 = b[9];
  double b2200 = b[10]; 
  double b3000 = b[11]; 
  double b0010 = b[12]; 
  double b0011 = b[13]; 
  double b0012 = b[14]; 
  double b0110 = b[15]; 
  double b0111 = b[16]; 
  double b0210 = b[17]; 
  double b1010 = b[18]; 
  double b1011 = b[19]; 
  double b0020 = b[20]; 
  double b0021 = b[21]; 
  double b0120 = b[22]; 
  double b1020 = b[23]; 
  double b0030 = b[24];

  double out[25] = { b0000, b0001, b0003, b0010, b0011, b0012, b0020, b0021,
          b0030, b0102, b0110, b0111, b0120, b0200, b0201, b0210, b1000, b1001, b1010, b1011, b1020,
          b1200, b2000, b2200, b3000};
  
  cout<<"Saving Vector: "<<endl;
  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<out[i]<<"  "<<endl;
     outFile<<out[i]<<" ";
  }
  outFile<<endl;
  outFile.close();
}


int ImportData(TMatrixD &A0, TVectorD &bVec0, TString filename, int NMax=10000000, int NumberOfMatrixElements = 25){
  int n0 = 0;
  ifstream myfile0(filename.Data());
  if (myfile0.is_open())
  {
    while (! myfile0.eof() && n0<NMax )
    {
      double x,y,th,ph, TgYC, rC, TgYReal;
      myfile0 >> x >> y >> th >> ph >> TgYC >> rC >> TgYReal;
      A0.ResizeTo(A0.GetNrows()+1, NumberOfMatrixElements);
      bVec0.ResizeTo(bVec0.GetNrows()+1);
      TVectorD line(NumberOfMatrixElements);
      
      line[0] = 1;
      line[1] = ph;
      line[2] = pow(ph,3);
      line[3] = pow(ph,2)*th;
      line[4] = pow(th,2);
      line[5] = ph*pow(th,2);
      line[6] = x;
      line[7] = ph*x;
      line[8] = pow(th,2)*x;
      line[9] = pow(x,2);
      line[10] = pow(th,2)*pow(x,2);
      line[11] = pow(x,3);
      line[12] = y;
      line[13] = ph*y;
      line[14] = pow(ph,2)*y;
      line[15] = th*y;
      line[16] = ph*th*y;
      line[17] = pow(th,2)*y;
      line[18] = x*y; 
      line[19] = ph*x*y;
      line[20] = pow(y,2);
      line[21] = ph*pow(y,2);
      line[22] = th*pow(y,2);
      line[23] = x*pow(y,2);
      line[24] = pow(y,3);

      TMatrixDRow(A0,A0.GetNrows()-1) = line;
      bVec0[bVec0.GetNrows()-1] = TgYC ;
      n0++;

      
    }
    myfile0.close();
    return n0-1;
  }
  else cout << "Unable to open file"; 
  return -1;

}





void TargetYSVD(int runnumber)
{
  const int NumberOfMatrixElements = 25;
  const int ConsideredMatrixElements = 25;
  const int NMax = 4000;
  TMatrixD A(0,0);
  TVectorD b(0);

  TString filename;
  filename.Form("OutFileTargetY0_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY1_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY2_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY3_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY4_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY5_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY6_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );

  
  A.Print();
  b.Print();

  

  TDecompSVD SVD(A,0.01);
/*
  SVD.Decompose();
  
  TMatrixD U = SVD.GetU();
  TMatrixD V = SVD.GetV();
  TVectorD sigma = SVD.GetSig();

  sigma.Print();
  // let's plot singular values
  double x[NumberOfMatrixElements];
  double y[NumberOfMatrixElements];
  double ey[NumberOfMatrixElements];
  double ex[NumberOfMatrixElements];
  for (int i = 0; i< NumberOfMatrixElements; i++){
    x[i] = i;
    y[i] = sigma[i];
  }
  TCanvas *c1 = new TCanvas("c1","c1");
  c1->cd(1);
  gPad->SetLogy();
  gPad->SetGrid();
  TGraph *gr1 = new TGraph(NumberOfMatrixElements, x,y);
  gr1->SetMarkerStyle(22);
  gr1->SetMarkerColor(kRed);
  gr1->Draw("AP");
  

  TVectorD aVec(NumberOfMatrixElements);
  TVectorD errorVecSq(NumberOfMatrixElements);
  
  for(int k = 0; k< ConsideredMatrixElements; k++ ){
      TVectorD UVec = TMatrixDColumn(U,k);
      TVectorD VVec = TMatrixDColumn(V,k);
      double sp = 0.0;
      for(int i = 0; i<UVec.GetNrows(); i++) sp+= UVec[i]*b[i]; 
      

      aVec = aVec + sp/sigma[k]*VVec;
      for(int i = 0; i< errorVecSq.GetNrows(); i++)errorVecSq[i] =  errorVecSq[i]+ pow(TMatrixDRow(V,i)[k]/sigma[k],2);

      x[k] = k;
      y[k] = aVec[7];     
      ex[k] = 0.0;
      ey[k] = sqrt(errorVecSq[7]) ;
  }

  errorVecSq.Print();

  TCanvas *c2 = new TCanvas("c2","c2");
  c2->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr2 = new TGraphErrors(NumberOfMatrixElements, x,y, ex, ey);
  gr2->SetMarkerStyle(23);
  gr2->SetMarkerColor(kRed);
  gr2->Draw("AP");
  
  TMatrixD Cov(NumberOfMatrixElements,NumberOfMatrixElements);
  for(int j=0; j<NumberOfMatrixElements; j++){
    for(int k=0; k<NumberOfMatrixElements; k++){
       double elementCov = 0;
       for( int i = 0; i <ConsideredMatrixElements ; i++){
          elementCov = elementCov + TMatrixDRow(V, k)[i]*TMatrixDRow(V, j)[i]/sigma[i]/sigma[i];
       }
       TMatrixDRow(Cov, j)[k] = elementCov;
     }
  }
  Cov.Print();
 
  TVectorD errorVec(NumberOfMatrixElements);
  for(int j = 0; j<NumberOfMatrixElements; j++){
        errorVec[j] = sqrt(TMatrixDRow(Cov, j)[j]);
   }
   errorVec.Print();
  //SVD.Solve(b);
 

  cout<<"Let's print out the vector y: "<<endl;
  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<aVec[i]<<" p/m  "<<errorVec[i]<<" or % "<<errorVec[i]/aVec[i]<<endl;
  }


 ExportData(aVec, "ParameterResultsTgYSVD.dat", NumberOfMatrixElements);
 TString name;
 name.Form("ParameterResultsTgYSVD_%d.dat", runnumber);
 ExportDisplayData(aVec, name.Data(), NumberOfMatrixElements, A.GetNrows());

*/

}

