#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph.h"
#include "TGraphErrors.h" 
#include "TROOT.h"
#include "TF1.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"
#include "TBox.h"


using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.17);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}






void TargetYSVDPlotResults()
{
  HallA_style();
  ifstream myfile0("MatrixElements.dat");
  if (myfile0.is_open())
  {
    while (!myfile0.eof())
    {
      TString MatrixElement;
      myfile0 >> MatrixElement;
      //MatrixElement = "b0010";
      TString filename;
      filename.Form("ValuesMatrixElement_%s.dat", MatrixElement.Data() );

      TGraph *grElement = new TGraph();
      TMatrixD elemb(0,0);
      ifstream myfile(filename.Data());
      if (myfile.is_open())
      {
        while (! myfile.eof())
        {
          double bval;
          myfile >> bval;
          elemb.ResizeTo(elemb.GetNrows()+1, 2);
          elemb(elemb.GetNrows()-1, 0) = bval;
          elemb(elemb.GetNrows()-1, 1) = 1.0;
          grElement->SetPoint(elemb.GetNrows()-1, elemb.GetNrows()-1, bval);
        }
        myfile.close();
      }
      else cout << "Unable to open file"; 
      //elemb.Print();
      

      double sum = 0.0;
      for(int i = 0; i<elemb.GetNrows(); i++) sum +=elemb(i, 0);
      double mean = sum/elemb.GetNrows();
 
      double sigmaSq = 0.0;
      for(int i = 0; i<elemb.GetNrows(); i++) sigmaSq += pow(elemb(i, 0) - mean,2);
      double sigma = sqrt(sigmaSq*1.0/(elemb.GetNrows()-1));
 
      cout<<"Mean Value of parameter "<<MatrixElement.Data()<<" : "<<mean<<" +/- "<<sigma<<", Relative Error: "<<fabs(sigma/mean)*100.0<<" [%]"<<endl; 

      TCanvas *c1 = new TCanvas("c1","c1",800,500);
      c1->cd(1);
      gPad->SetGrid();
      grElement->Draw("APT");
      grElement->SetMarkerColor(kRed);
      grElement->SetMarkerStyle(20);
      filename.Form("Matrix Element %s : %f +/- %f",MatrixElement.Data(), mean, sigma );
      grElement->SetTitle(filename.Data());
      grElement->GetXaxis()->SetTitle("Run #");
      grElement->GetYaxis()->SetTitle(MatrixElement.Data());

      TLine *LineMean = new TLine(0.0, mean, elemb.GetNrows()+1, mean);
      LineMean->SetLineColor(kGreen);
      LineMean->SetLineWidth(2.0);
      LineMean->Draw("same");

      TBox *OneSigmaRegion = new TBox(0, mean - sigma, elemb.GetNrows()+1, mean+sigma );
      OneSigmaRegion->SetLineWidth(2.0);
      OneSigmaRegion->SetLineColor(kGreen);
      OneSigmaRegion->SetFillStyle(3003);
      OneSigmaRegion->SetFillColor(kGreen);
      OneSigmaRegion->Draw("same");
      grElement->Draw("PT");

      filename.Form("figure_TgY_MatrixElement_%s.png", MatrixElement.Data());
      c1->Print(filename.Data());
      //gr->Delete();

     


    }
    myfile0.close();
  }
  else cout << "Unable to open file"; 
}

