#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TGraph.h"
#include "TGraphErrors.h" 
#include "TROOT.h"
#include "TF1.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"
#include "TVector.h"

using namespace std;

void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);

  gStyle->SetOptStat(0);

}

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
        (maxarg1) : (maxarg2))

static int iminarg1,iminarg2;
#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
        (iminarg1) : (iminarg2))

float pythag(float a, float b)
{
	float absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+pow((absb/absa),2));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+pow((absa/absb),2)));
}

//void svdcmp(float **a, int m, int n, float w[], float **v)
void svdcmp(TMatrixF &a, int m, int n, TVectorF &w, TMatrixF &v)
{
	int flag,i,its,j,jj,k,l,nm;
	float anorm,c,f,g,h,s,scale,x,y,z;//,*rv;

	TVectorF rv1(1,n);
	g=scale=anorm=0.0;
	for (i=1;i<=n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<=m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<=m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=i;k<=m;k++) s += a[k][i]*a[k][j];
					f=s/h;
					for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
				}
				for (k=i;k<=m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<=n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<=n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<=n;k++) rv1[k]=a[i][k]/h;
				for (j=l;j<=m;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[j][k]*a[i][k];
					for (k=l;k<=n;k++) a[j][k] += s*rv1[k];
				}
				for (k=l;k<=n;k++) a[i][k] *= scale;
			}
		}
		anorm=FMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
        cout<<"burek"<<endl;
	for (i=n;i>=1;i--) {
		if (i < n) {
			if (g) {
				for (j=l;j<=n;j++)
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<=n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<=n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=IMIN(m,n);i>=1;i--) {
		l=i+1;
		g=w[i];
		for (j=l;j<=n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			for (j=l;j<=n;j++) {
				for (s=0.0,k=l;k<=m;k++) s += a[k][i]*a[k][j];
				f=(s/a[i][i])*g;
				for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
			}
			for (j=i;j<=m;j++) a[j][i] *= g;
		} else for (j=i;j<=m;j++) a[j][i]=0.0;
		++a[i][i];
	}
	for (k=n;k>=1;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=1;l--) {
				nm=l-1;
				if ((float)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((float)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((float)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=pythag(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=1;j<=m;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=1;j<=n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			if (its == 30) cerr<<"no convergence in 30 svdcmp iterations"<<endl;
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=pythag(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=pythag(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=1;jj<=n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=pythag(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=1;jj<=m;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
			//cout<<"Tle: "<<x<<",  "<<w[k]<<endl;
                        
		}
	}
	//free_vector(rv1,1,n);
}


void ExportData(TVectorF &b, TString filename, int NumberOfMatrixElements = 19 ){
 
  ofstream outFile;
  outFile.open(filename.Data());


  double b0000 = b[1]; 
  double b0001 = b[2]; 
  double b0002 = b[3]; 
  double b0003 = b[4]; 
  double b0004 = b[5]; 
  double b0010 = b[6]; 
  double b0011 = b[7]; 
  double b0012 = b[8]; 
  double b0013 = b[9]; 
  double b0022 = b[10];
  double b0031 = b[11]; 
  double b0100 = b[12]; 
  double b0102 = b[13]; 
  double b0110 = b[14]; 
  double b0111 = b[15]; 
  double b1002 = b[16]; 
  double b1010 = b[17]; 
  double b1011 = b[18]; 
  double b1020 = b[19]; 


  double out[19] = { b0000, b0001, b0002, b0003, b0004, b0010, b0011, b0012, b0013, b0022, b0031, b0100, b0102, b0110, b0111, b1002, b1010, b1011, b1020};

  cout<<"Saving Vector: "<<endl;
  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<out[i]<<"  "<<endl;
     outFile<<out[i]<<" ";
  }
  outFile<<endl;
  outFile.close();
}



void ExportDisplayData(TVectorF &b, TString filename, int NumberOfMatrixElements, int NumberOfPoints){
 
  ofstream outFile;
  outFile.open(filename.Data());

  double b0000 = b[1]; 
  double b0001 = b[2]; 
  double b0002 = b[3]; 
  double b0003 = b[4]; 
  double b0004 = b[5]; 
  double b0010 = b[6]; 
  double b0011 = b[7]; 
  double b0012 = b[8]; 
  double b0013 = b[9]; 
  double b0022 = b[10];
  double b0031 = b[11]; 
  double b0100 = b[12]; 
  double b0102 = b[13]; 
  double b0110 = b[14]; 
  double b0111 = b[15]; 
  double b1002 = b[16]; 
  double b1010 = b[17]; 
  double b1011 = b[18]; 
  double b1020 = b[19]; 


  double out[19] = { b0000, b0001, b0002, b0003, b0004, b0010, b0011, b0012, b0013, b0022, b0031, b0100, b0102, b0110, b0111, b1002, b1010, b1011, b1020};

  TString labels[19] = { "b0000", "b0001", "b0002", "b0003", "b0004", "b0010", "b0011", "b0012", "b0013", "b0022", "b0031", "b0100", "b0102", "b0110", "b0111", "b1002", "b1010", "b1011", "b1020"};
  


  cout<<"Saving Vector: "<<endl;

  cout<<"N: "<<NumberOfPoints<<endl;
  outFile<<"N: "<<NumberOfPoints<<endl;

  for (int i = 0; i<NumberOfMatrixElements; i++){
     cout<<labels[i]<<"  "<<out[i]<<" "<<endl;
     outFile<<labels[i]<<"  "<<out[i]<<" "<<endl;
  }
  outFile.close();
}


int ImportData(TMatrixF &A0, TVectorF &bVec0, TString filename, int NMax=10000000, int NumberOfMatrixElements = 19){

  int n0 = 0;
  ifstream myfile0(filename.Data());
  if (myfile0.is_open())
  {
    while (! myfile0.eof() && n0<NMax )
    {
      n0++;
      float x,y,th,ph, TgYC, rC, TgYReal;
      myfile0 >> x >> y >> th >> ph >> TgYC >> rC >> TgYReal;
      A0.ResizeTo(A0.GetNrows()+1, NumberOfMatrixElements+1);
      bVec0.ResizeTo(bVec0.GetNrows()+1);
     
      A0[A0.GetNrows()-1][0] = 0;
      A0[A0.GetNrows()-1][1] = 1;
      A0[A0.GetNrows()-1][2] = ph;
      A0[A0.GetNrows()-1][3] = pow(ph,2);
      A0[A0.GetNrows()-1][4] = pow(ph,3);
      A0[A0.GetNrows()-1][5] = pow(ph,4);
      A0[A0.GetNrows()-1][6] = y;
      A0[A0.GetNrows()-1][7] = y*ph;
      A0[A0.GetNrows()-1][8] = y*pow(ph,2);
      A0[A0.GetNrows()-1][9] = y*pow(ph,3);
      A0[A0.GetNrows()-1][10] = pow(y,2)*pow(ph,2);
      A0[A0.GetNrows()-1][11] = pow(y,3)*pow(ph,1);
      A0[A0.GetNrows()-1][12] = th;
      A0[A0.GetNrows()-1][13] = th*pow(ph,2);
      A0[A0.GetNrows()-1][14] = th*y;
      A0[A0.GetNrows()-1][15] = th*y*ph;
      A0[A0.GetNrows()-1][16] = x*pow(ph,2);
      A0[A0.GetNrows()-1][17] = x*y;
      A0[A0.GetNrows()-1][18] = x*y*ph;
      A0[A0.GetNrows()-1][19] = x*pow(y,2); 


      bVec0[bVec0.GetNrows()-1] = TgYC;
      
    }
    myfile0.close();
    return bVec0.GetNrows();
  }
  else cout << "Unable to open file"; 
  return -1;

}





void TargetYSVDrawNewBest(int runnumber)
{
  const int NumberOfMatrixElements = 19;
  const int ConsideredMatrixElements = 19;
  const int NMax = 50000;
  TMatrixF A(1,1);
  TVectorF b(1);

  TString filename;
  filename.Form("OutFileTargetY0_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY1_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY2_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY3_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY4_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY5_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY6_%d.dat", runnumber);
  ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );
  filename.Form("OutFileTargetY7_%d.dat", runnumber);
  int DataNum = ImportData(A, b, filename.Data(), NMax,NumberOfMatrixElements  );

  cout<<"Data Loaded! "<<DataNum<<endl;
  //A.Print();
  //b.Print();

  TMatrixF V(A.GetNcols(), A.GetNcols());
  TVectorF w(A.GetNcols());

  TMatrixF U = A;
  svdcmp( U, U.GetNrows()-1, NumberOfMatrixElements, w, V);
  
  w.Print();
  
 
  // let's plot singular values
  double x[NumberOfMatrixElements+1];
  double y[NumberOfMatrixElements+1];
  double ey[NumberOfMatrixElements+1];
  double ex[NumberOfMatrixElements+1];
  for (int i = 1; i<=NumberOfMatrixElements; i++){
    x[i] = i;
    y[i] = w[i];
  }

  TCanvas *c1 = new TCanvas("c1","c1");
  c1->cd(1);
  gPad->SetLogy();
  gPad->SetGrid();
  TGraph *gr1 = new TGraph(NumberOfMatrixElements, x,y);
  gr1->SetMarkerStyle(22);
  gr1->SetMarkerColor(kRed);
  gr1->Draw("AP");
  

  TVectorF aVec(NumberOfMatrixElements+1);
  TVectorF errorVecSq(NumberOfMatrixElements+1);
  
  for(int k = 1; k<=ConsideredMatrixElements; k++ ){
      TVectorF UVec = TMatrixFColumn(U,k);
      TVectorF VVec = TMatrixFColumn(V,k);
      double sp = 0.0;
      for(int i = 1; i<=UVec.GetNrows()-1; i++) sp+= UVec[i]*b[i]; 
      

      for (int i = 1; i<=NumberOfMatrixElements; i++) aVec[i] = aVec[i] + sp/w[k]*VVec[i];
      for(int i = 1; i<=errorVecSq.GetNrows()-1; i++)errorVecSq[i] =  errorVecSq[i]+ pow(V[i][k]/w[k],2);

      x[k] = k;
      y[k] = aVec[7];     
      ex[k] = 0.0;
      ey[k] = sqrt(errorVecSq[7]) ;
  }

  errorVecSq.Print();

  TCanvas *c2 = new TCanvas("c2","c2");
  c2->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr2 = new TGraphErrors(NumberOfMatrixElements, x,y, ex, ey);
  gr2->SetMarkerStyle(23);
  gr2->SetMarkerColor(kRed);
  gr2->Draw("AP");
  

  TVectorD errorVec(NumberOfMatrixElements+1);
  for(int j = 1; j<=NumberOfMatrixElements+1; j++){
        errorVec[j] = sqrt(errorVecSq[j]);
   }
   errorVec.Print();
  //SVD.Solve(b);
 
  cout<<"Let's print out the vector y: "<<endl;
  for (int i = 1; i<=NumberOfMatrixElements; i++){
     cout<<aVec[i]<<" p/m  "<<errorVec[i]<<" or % "<<errorVec[i]/aVec[i]<<endl;
     //cout<<aVec[i]<<endl;
  }

 TString name;

 ExportData(aVec, "ParameterResultsTgYSVD.dat", NumberOfMatrixElements);

 name.Form("ParameterResultsTgYSVD_%d_New.dat", runnumber);
 ExportDisplayData(aVec, name.Data(), NumberOfMatrixElements, A.GetNrows());

}

